 /*****************************************************************************
 *                                                                           *
 * Script file for Sound Forge, version 9.0 - 11.0                                 *

 *                                                                           *
 *****************************************************************************/

/*************************************
 * USES                              *
 *************************************/
use "ForgeLib.jsb"
use "ForgeBrl.jsb"
use "ForgeSay.jsb"
use "ForgePeak.jsb"
use "ForgeMarker.jsb"
use "forgetrack.jsb"
/*************************************
 * STANDARD INCLUDES                 *
 *************************************/
include "common.jsm"
include "hjconst.jsh"
include "hjglobal.jsh"

/*************************************
 * Sound Forge specific includes          *
 *************************************/
include "forge.jsm"
Include "forge.jsh"
Include "license.jsh"

; Hotkeys:
; Reading commands:
; Read current time (selection start) -> JawsKey + C
; Read total time -> JawsKey + L
; Read selection start -> JawsKey + S
; Read selection start & end -> Ctrl + JawsKey + S
; Read active channels -> Jawskey+n
; Speak track info -> Ctrl + Shift + I
;
; Toggle commands:
; Toggle speech on/off -> Jawskey + F12
; Loop left -> F11
; Loop right -> F12
; Clipping alarm on/off -> JawsKey + A
;
; List commands:
; Select a marker -> JawsKey + M
; Select a peak meter to clear -> JawsKey + P
; List graphical buttons -> Ctrl + *Shift + G
; Select plug-in tabs -> Ctrl + Shift + P
; Select a tab in a multi-page dialog -> Ctrl + Shift + F10
;
; SF standard commands:
; Change zoom ratio -> arrow up / arrow down
; Activate next channel -> Tab
; Activate prev channel -> Shift+Tab
;
; Tracks and markers:
; Show a menu for a marker -> Second routing key on the marker or Alt + F10
; Clear a marker from a list -> Ctrl+Shift+p
; Move to a marker -> CR on the marker in marker mode
; Delete track or marker -> Alt + Delete
;
; Misc:
; Clear a peak meter -> CR on the peak meter to clear in peak meters mode
; Move between modes -> EAB up/down
; Move to a location: -> CR on a location in the marker mode

; Requirements:
; JAWS v10.0.512U or later
; Color depth: 32 bits
; Resolution: 1280x1024

; Known problems:
; * Resize peak meters window with SendMessage

; Todo
; * Handle the class SfKrn_Spread_Class
; This is a Excel-like spread
; Could be found in Process/channel converter
; * Read peak-meters with hotkey
; * A possibility to check if loop playing is on or off
; Options/Loop playback (Alt+o,b)
; A command to check the status would help
; * Record dialog
; When selecting Remote a totally different dialog is shown
; This is not accessible with the scripts

; Noise Reduction:
; Wrong prompts are read for two of the edit fields

; * History
; Version 0.9, Dec 2009:
; First official version of the scripts
; Works with Sound Forge 9.0e
; Version 1.0, Feb 2010:
; Works with Sound Forge 9.0e and Sound Forge 10
; A lot of fixes
; Version 1.1, Oct 2010:
; Works with Sound Forge 9.0e and Sound Forge 10
; A lot of fixes
; Version 1.2, Nov 2010:
; Works with Sound Forge 9.0e and Sound Forge 10
; A lot of fixes
; New license system
; New work-mode (channel mode)
; Better handling of the record dialog
; Version 1.5, Nov 2015:
; Works with Sound Forge 11.0

/*****************************************************************************
 *                                                                           *
 * Overridden System Functions and Scripts                                   *
 *                                                                           *
 *****************************************************************************/

/******************
 * Description:
 *      Overrided this function to handle initializations.
 * Return:
 *      Void
 * Param:
 *      -
 * Note:
 *      For more information read the FSDN help.
 ******************/
void Function AutoStartEvent ()
	var handle hWnd
	; Load license
	let gbChecked = LoadLicense ()

; Check license
	CheckLicense ()

	if (not bHasRunBefore) then
		; Do this the first time
		SayMessage (OT_APP_START,msgDevelopedBy,msgDevelopedBy)
 		SayMessage (OT_APP_START,msgVersion,msgVersion)
		let bHasRunBefore = TRUE
		; Check system requirements
;		if (Not CheckRequirements ()) then
;			SayMessage (OT_JAWS_MESSAGE,msgSystemReq,msgSystemReq)
;		EndIf
	EndIf
	; Do initialization
	ScheduleFunction ("Setup",5)

	; Read .ini-file
	ReadIniFile()

	; Start monitoring the timers
	StartTimer ()
	; Set Braille mode to time mode
	let giMode = TIME
	; Store the dots of the braille cursor
	let gsBrailleCursorDots = BrailleGetCursorDots ()
EndFunction

/******************
 * Description:
 *      -
 * Return:
 *      Void
 * Param:
 *      -
 * Note:
 *      For more information read the FSDN help.
 ******************/
Void Function AutoFinishEvent ()
	; Stop the timer function
	UnScheduleFunction (giTimer)
EndFunction

/******************
 * Description:
 *      Overrided this function to be able to initialize focus in the record dialog
 * Return:
 *      Int (used as boolean), 
 * Param:
 *      Handle, the handle for MDI window.
 * Note:
 *      For more information read the FSDN help.
 ******************/
int Function HandleCustomRealWindows (handle hWnd)
	var handle hFocusWnd
	if (IsRecordDlg()) then
		; The record dialog is open
		; Set focus to the New button
		let hFocusWnd = FindDescendantWindow (hwnd,ciNewCID)
		SetFocus (hFocusWnd)
		; Turn of autopanning
		SetJcfOption (OPT_BRL_AUTOPAN_MODE,brlAutoPanOff)
		BraillePanLeft()
		return TRUE
	EndIf
	return FALSE	
EndFunction

/******************
 * Description:
 *      Overrided this function to be able enhance window handling on windows that needs improvement.
 * Return:
 *      Int (used as boolean), 
 * Param:
 *      Handle, the handle for current window.
 * Note:
 *      For more information read the FSDN help.
 ******************/
int Function HandleCustomWindows (handle hWnd)
	if (IsDataWindow()) then
		; Focus is in the data window
		; Maximize the data window
		SendMessage (GetFocus (),WM_SYSCOMMAND, SC_MAXIMIZE,0)
		if (GetJcfOption (Opt_BRL_AUTOPAN_MODE) != brlAutoPanOff) then
			; Set the correct Braille panning options to show the structured info
			SetJcfOption (OPT_BRL_AUTOPAN_MODE,brlAutoPanOff)
			BrailleSetCursorDots ("")
		EndIf
		BraillePanLeft()
		if (bUpdateMarkers) then
			; Markers need to be updated
			if (not gbClosed) then
				; Close the marker list,
				CloseListWindows ()
				let gbClosed = TRUE
			EndIf
			let bUpdateMarkers = FALSE
			; Update the marker info
			ScheduleFunction("UpdateMarkers",5)
		EndIf
		PerformScript SayMode ()

		; Get info about active channels
		let gsChannelInfo = GetChannelInfo ()
		PerformScript SayChannels ()
		return TRUE
	ElIf (IsFilePropertiesWindow ()) then
		{LeftArrow}
		{LeftArrow}
		return
	Else
		if (BUpdateMarkers && GetControlID(GetFocus()) == ciMarkersAndRegionsListCID ) then
			; The markers and regions list is open and need to be closed
			CloseListWindows ()
			let gbClosed = TRUE
		EndIf
		; Set the Braille panning options to follow the focus
		if (not IsRecordDlg () && (GetJcfOption(Opt_BRL_AUTOPAN_MODE) != brlAutoPanSmart)) then
			SetJcfOption (OPT_BRL_AUTOPAN_MODE,brlAutoPanSmart)
		EndIf
		; Show the braille cursor
		BrailleSetCursorDots (gsBrailleCursorDots)
		; Set the Braille panning options to follow the focus
		if (not IsRecordDlg () && (GetJcfOption(Opt_BRL_AUTOPAN_MODE) != brlAutoPanSmart)) then
			SetJcfOption (OPT_BRL_AUTOPAN_MODE,brlAutoPanSmart)
		EndIf
		if ((GetWindowClass(hwnd) == csForgeButtonsClass) && (GetControlID(hWnd) == 4361)) then
			if (IsWindowDisabled (GetPriorWindow (hWnd))) then
				IndicateControlType (WT_BUTTON, msgUnLock,"")
				let gsButtonValue = msgUnLock
			else
				IndicateControlType (WT_BUTTON, msgLock,"")
				let gsButtonValue = msgLock
			EndIf
			return
		Elif (IsRecordSpecialButton (hWnd)) then
				;	Focus is on one of the non-standard buttons in the record dialog
				HandleRecButtons ()
				return TRUE
		ElIf (GetWindowClass (hWnd) == csSFSpread)then
			SayString ("Sound forge spread")
			return TRUE
		ElIf (IsSFUpDownSlider(hWnd)) then
			IndicateControlType (WT_UPDOWNSLIDER, GetSFUpDownSliderLabel(hWnd),GetSFUpDownSliderValue(hWnd))
			return TRUE
		EndIf
	EndIf
	; Default handling;		let hFocusWnd = FindDescendantWindow(hWnd,4023)
	HandleCustomWindows (hWnd)
EndFunction

/******************
 * Description:
 *      Handles the special buttons in the record dialog
 * Return:
 *      -
 * Param:
 *      -
 * Note:
 *      Dependent of pixel positions
 ******************/
Function HandleRecButtons ()
	var int iFocusRectLeft
	let iFocusRectLeft = GetFocusRectLeft (GetFocus()) - GetWindowLeft(GetFocus ())
	DebugMessage (IntToString(iFocusRectLeft),0)
	if (iFocusRectLeft == StringToInt(StringSegment(gsRecButtonPixels,"|",1))) then
		let gsButtonValue = msgArm
	Elif (iFocusRectLeft == StringToInt(StringSegment(gsRecButtonPixels,"|",2))) then
		if (not IsRecording()) then
			let gsButtonValue = msgRecord
		Else
			let gsButtonValue = msgStop
		EndIf
	Elif (iFocusRectLeft == StringToInt(StringSegment(gsRecButtonPixels,"|",3))) then
		let gsButtonValue = msgPlay
	Elif (iFocusRectLeft == StringToInt(StringSegment(gsRecButtonPixels,"|",4))) then
		let gsButtonValue = msgDropMarker
	Elif (iFocusRectLeft == StringToInt(StringSegment(gsRecButtonPixels,"|",5))) then
		let gsButtonValue = msgGoToStart
	Elif (iFocusRectLeft == StringToInt(StringSegment(gsRecButtonPixels,"|",6))) then
		let gsButtonValue = msgGotoStartOfLastTake
	Elif (iFocusRectLeft == StringToInt(StringSegment(gsRecButtonPixels,"|",7))) then
		let gsButtonValue = msgGotoEnd
	Elif (iFocusRectLeft == StringToInt(StringSegment(gsRecButtonPixels,"|",8))) then
		let gsButtonValue = msgGoto
	EndIf
	IndicateControlType (WT_BUTTON, gsButtonValue, "")
EndFunction

/******************
 * Description:
 *      Overwritten to catch some keystrokes
 * Return:
 *      -
 * Param:
 *      -
 * Note:
 *      For more information read the FSDN help.
 ******************/
Void Function KeyPressedEvent (int nKey, string strKeyName, int nIsBrailleKey, int nIsScriptKey)
	; Clear Braille info message
	let giInfo = FALSE

 	if ((giMode == MARKERS) && IsDataWindow () && strKeyName == "m") then
			; The user inserts a marker
			ScheduleFunction("UpdateMarkers",5)
 	Elif ((giMode == TRACKS) && IsDataWindow () && strKeyName == "n") then
			; The user inserts a track
			ScheduleFunction("UpdateTracks",5)
	EndIf
	if ((not IsLoopstartKey (strKeyName)) && (not IsMovementKey (strKeyName))) then
		; Turn off looping
  		if (giTrim == 1) then
			PerformScript ToggleLoopRight ()
		ElIf (giTrim == 2) then
			PerformScript ToggleLoopLeft ()
		EndIf		
	EndIf
	; Call default
	KeyPressedEvent (nKey,strKeyName, nIsBrailleKey, nIsScriptKey)
EndFunction

/******************
 * Description:
 *      Overrided this function to do special handling when the menu gets focus
 * Return:
 *      -
 * Param:
 *      handle WinHandle, The handle of the menu
* 					 int mode, The menu mode
* Note:
 *      For more information read the FSDN help.
 ******************/
Void Function MenuModeEvent (handle WinHandle, int mode)
	if (mode != MENU_InACTIVE) then
		; The menu is closed
		if (GetJcfOption (Opt_BRL_AUTOPAN_MODE) != brlAutoPanSmart) then
			; Set the Braille to follow the focus
			SetJcfOption (OPT_BRL_AUTOPAN_MODE,brlAutoPanSmart)
		EndIf
	EndIf
	; Call the defaults
	MenuModeEvent (WinHandle, mode)
EndFunction

/*************************************
 * Scripts                           *
*************************************/

/******************
 * Description:
 *      Overrided this function to get reading of zoom ratio in the data window and to read SF sliders
 * Return:
 *      -
 * Param:
 *      -
 * Note:
 *      For more information read the FSDN help.
******************/
Script SayNextLine ()
	var string sZoomRatio

	; Check license
	CheckLicense ()

	if (IsDataWindow () && IsPcCursor () && (not IsVirtualPcCursor ())) then
		NextLine ()
		let sZoomRatio = GetZoomRatio ()
		SayString (sZoomRatio)
		BrailleMessage (sZoomRatio, 0, BRAILLEMESSAGEDURATION)	
	ElIf (IsPcCursor () && IsSFUpDownSlider(GetFocus ())) then
		; SF slider
		NextLine ()
		SayString (GetSFUpDownSliderValue(GetFocus()))
	else
		PerformScript SayNextLine ()
	EndIf
EndScript

/******************
 * Description:
 *      Overrided this function to read zoom ratio in the data window and to read SF sliders
 * Return:
 *      -
 * Param:
 *      -
 * Note:
 *      For more information read the FSDN help.
******************/
Script SayPriorLine ()
	var string sZoomRatio
	; Check License
	CheckLicense ()

	if (IsDataWindow () && isPcCursor () && (not IsVirtualPcCursor ())) then
		PriorLine ()
		let sZoomRatio = GetZoomRatio ()
		SayString(sZoomRatio)
		BrailleMessage (sZoomRatio, 0, BRAILLEMESSAGEDURATION)	
	ElIf (IsPcCursor () && IsSFUpDownSlider(GetFocus ())) then
		; SF slider
		PriorLine ()
		SayString (GetSFUpDownSliderValue(GetFocus()))
	else
		PerformScript SayPriorLine ()
	EndIf
EndScript

/******************
 * Description:
 *      Overrided this function to get autolooping to work
 * Return:
 *      -
 * Param:
 *      -
 * Note:
 *      For more information read the FSDN help.
 ******************/
Script SayNextCharacter ()
	; Check license
	CheckLicense()
	if (not IsVirtualPcCursor () && IsDataWindow ()) then
		if (giTrim) then
			; we're in autoloop mode, so stop playing
			{Space}
		EndIf
		NextCharacter ()
		if (giTrim) then
			; We're in autoloop mode so start playing again
			{Space}
		else
			let gsTimers = GetTimers ()
			PerformScript SpeakCurrentTime ()	
		EndIf
	else
		PerformScript SayNextCharacter ()
	EndIf
EndScript

/******************
 * Description:
 *      Overrided this function to get autolooping to work
 * Return:
 *      -
 * Param:
 *      -
 * Note:
 *      For more information read the FSDN help.
 ******************/
Script SayPriorCharacter ()
	; Check license
	CheckLicense ()
	if (not IsVirtualPcCursor () && IsDataWindow ()) then
		if (giTrim) then
			; We're in autoloop mode, so stop playing
			{Space}
		EndIf
		PriorCharacter ()
		if (giTrim) then
			; We're in autoloop mode so start playing again
			{Space}
		else
			let gsTimers = GetTimers ()
			PerformScript SpeakCurrentTime ()	
		EndIf
	else
		PerformScript SayPriorCharacter ()
	EndIf
EndScript

/******************
 * Description:
 *      Overrided this function to make it possible to close the markers and regions list with Ctrl+F4
 * Return:
 *      -
 * Param:
 *      -
 * Note:
 *      For more information read the FSDN help.
 ******************/
Script CloseWindow ()
	var string sClass,
	handle hRealWnd
	let hRealWnd = GetRealWindow (GetFocus ())
	let sClass = GetWindowClass (hRealWnd)
	if (StringContains (sClass,csSF90DialogClass) || StringContains(sClass,csSF10DialogClass)) then
		SendMessage(hRealWnd, WM_CLOSE,0 , 0)
	else
		; Do the standard
		{Ctrl+F4}
	EndIf
EndScript

/******************
 * Description:
 *      Overrided this function to get screen sensitive help in Sound Forge
 * Return:
 *      -
 * Param:
 *      -
 * Note:
 *      For more information read the FSDN help.
 ******************/
Script ScreenSensitiveHelp ()
	if (IsDataWindow ()) then
		SayFormattedMessage (OT_USER_BUFFER,msgHelpDataWindow,msgHelpDataWindow)
		AddHotKeyLinks ()
		return
	EndIf
	; Default handling
	PerformScript ScreenSensitiveHelp ()
EndScript

/******************
 * Description:
 *      Overrided this function to get hotkey help in Sound Forge
 * Return:
 *      -
 * Param:
 *      -
 * Note:
 *      For more information read the FSDN help.
 ******************/
Script HotKeyHelp ()
	if (IsDataWindow()) then
		SayFormattedMessage (OT_USER_BUFFER,msgHotkeys,msgHotkeys)
		return
	EndIf
	PerformScript HotKeyHelp ()
EndScript

/******************
 * Description:
 *      Overrided this function to get reading of sliders and movement in the data window to work
 * Return:
 *      -
 * Param:
 *      -
 * Note:
 *      For more information read the FSDN help.
 ******************/
Script JAWSPageDown ()
	PerformScript JawsPageDown ()
	if (IsVirtualPcCursor ()) then
		return
	EndIf
	if (IsDataWindow ()) then
		let gsTimers = GetTimers ()
		PerformScript SpeakCurrentTime ()
	Elif (IsSFUPDownSlider(GetFocus ())) then
		SayString(GetSFUpDownSliderValue (GetFocus ()))
	EndIf
EndScript

/******************
 * Description:
 *      Overrided this function to get reading of sliders and movement in the data window to work
 * Return:
 *      -
 * Param:
 *      -
 * Note:
 *      For more information read the FSDN help.
 ******************/
Script JAWSPageUp ()
	PerformScript JawsPageUp ()
	if (IsVirtualPcCursor ()) then
		return
	EndIf
	if (IsDataWindow ()) then
		let gsTimers = GetTimers ()
		PerformScript SpeakCurrentTime ()	
	Elif (IsSFUPDownSlider(GetFocus ())) then
		SayString(GetSFUpDownSliderValue (GetFocus ()))
	EndIf
EndScript

/******************
 * Description:
 *      Overrided this function to get reading of sliders and movement in the data window to work
 * Return:
 *      -
 * Param:
 *      -
 * Note:
 *      For more information read the FSDN help.
 ******************/
Script JAWSHome ()
	PerformScript JawsHome ()
	if (not ISPcCursor ()) then
		return
	EndIf
	Pause ()
	if (IsDataWindow () && (not IsVirtualPCCursor ())) then
		let gsTimers = GetTimers()
		PerformScript SpeakCurrentTime()
	Elif (IsSFUPDownSlider(GetFocus ())) then
		SayString(GetSFUpDownSliderValue (GetFocus ()))
	EndIf
EndScript

/******************
 * Description:
 *      Overrided this function to get reading of sliders and movement in the data window to work
 * Return:
 *      -
 * Param:
 *      -
 * Note:
 *      For more information read the FSDN help.
 ******************/
Script JAWSEnd ()
	PerformScript JawsEnd ()
	Pause ()
	if (not IsPcCursor ()) then
		return
	EndIf
	if (IsDataWindow () && (not IsVirtualPCCursor ())) then
		let gsTimers = GetTimers()
		PerformScript SpeakCurrentTime()
	Elif (IsSFUPDownSlider(GetFocus ())) then
		SayString(GetSFUpDownSliderValue (GetFocus ()))
	EndIf
EndScript

/******************
 * Description:
 *      Overrided this function to get reading of sliders and movement in the data window to work
 * Return:
 *      -
 * Param:
 *      -
 * Note:
 *      For more information read the FSDN help.
 ******************/
script TopOfFile ()
	if (IsDataWindow () && IsPCCursor () && (not IsVirtualPcCursor ())) then
		SayMessage(ot_JAWS_message, cmsg36_L, cmsg36_S) ; top of file
		TypeKey(GetCurrentScriptKeyName ())
		Pause ()
		let gsTimers = GetTimers()
		PerformScript SpeakCurrentTime()
	Elif (IsSFUPDownSlider(GetFocus ())) then
		TypeKey(GetCurrentScriptKeyName ())
		Pause ()
		SayString(GetSFUpDownSliderValue (GetFocus ()))
	Else
		PerformScript TopOfFile ()
	EndIf
EndScript

/******************
 * Description:
 *      Overrided this function to get reading of sliders and movement in the data window to work
 * Return:
 *      -
 * Param:
 *      -
 * Note:
 *      For more information read the FSDN help.
 ******************/
script BottomOfFile ()
	if (IsDataWindow () && IsPCCursor () && (not IsVirtualPcCursor ()) && (not IsVirtualPcCursor ())) then
		SayMessage(ot_JAWS_message, cmsg37_L, cmsg37_S) ; bottom of file
		TypeKey(GetCurrentScriptKeyName ())
		Pause ()
		let gsTimers = GetTimers()
		PerformScript SpeakCurrentTime ()
	Elif (IsSFUPDownSlider(GetFocus ())) then
		TypeKey(GetCurrentScriptKeyName ())
		Pause ()
		SayString(GetSFUpDownSliderValue (GetFocus ()))
	Else
		PerformScript BottomOfFile ()
	EndIf
EndScript

/******************
 * Description:
 *      Give speech when navigating in the data window and when moving sliders
 * Return:
 *      -
 * Param:
 *      -
 * Note:
 *      - 
 ******************/
Script Navigate ()
	TypeKey(GetCurrentScriptKeyName ())
	Pause ()
	if (not IsPcCursor ()) then
		return
	EndIf
	if (IsDataWindow () && (not IsVirtualPcCursor ())) then
		let gsTimers = GetTimers ()
		Pause ()
		PerformScript SpeakCurrentTime ()	
	Elif (IsSFUPDownSlider(GetFocus ())) then
		SayString(GetSFUpDownSliderValue (GetFocus ()))
	EndIf
EndScript

Script SayNextWord ()
	var string sMarker, string sTrack
	if (IsPcCursor () && (not IsVirtualPcCursor ()) && IsDataWindow ()) then
		NextWord ()
		Pause () 
		let gsTimers = GetTimers ()
		PerformScript SpeakCurrentTime ()	
		ScheduleFunction ("SpeakMarkerAndTrack",10)
	else
		PerformScript SayNextWord ()
	EndIf
EndScript

Script SayPriorWord ()
	var string sMarker, string sTrack
	if (IsPcCursor () && (not IsVirtualPcCursor ()) && IsDataWindow ()) then
		PriorWord ()
		Pause ()
		let gsTimers = GetTimers ()
		PerformScript SpeakCurrentTime ()	
		ScheduleFunction ("SpeakMarkerAndTrack",10	)
	else
		PerformScript SayPriorWord ()
	EndIf
EndScript

; Speaks marker or track when navigating
Function SpeakMarkerAndTrack ()
	var string sMarker, string sTrack
	let sMarker = GetCurrentMarker ()
	SayString (sMarker)
	if (not sMarker) then
		let sTrack = GetCurrentTrack ()
		SayString(msgTrack + sTrack)
	EndIf
EndFunction

/******************
 * Description:
 *      Overrided this script to get a Tab order in the record dialog
 * Return:
 *      -
 * Param:
 *      -
 * Note:
 *      For more information read the FSDN help.
 ******************/
Script Tab ()
	var int iCurFocus, 	handle hCurFocus, handle hTmp, int iFocusRectLeft, string sCurTab,
	handle hRecDlg, string sApp, string sTmpChannels, int iNoOfChannels
	let sApp = GetAppFileName ()
	if (IsRecordDlg ()) then
		; The record dialog is active
		let iCurFocus = GetControlID (GetFocus ())
		let hCurFocus = GetFocus ()
		let hTmp = hCurFocus
		let sCurTab = GetWindowName (FindDescendantWindow(GetParent(GetFocus ()),4113))
		let hRecDlg = GetRealWindow (GetFocus ())
		if ((sApp == csForge90) && (iCurFocus == ciRecMetersTabCID) && (sCurTab == scMetersTab)) then
			; Sound Forge 9 and focus is on the Meters tab
			; Set focus to the first control in the tab
			SetFocus (FindDescendantWindow(GetParent(hCurFocus),ciMonitorCID))
			return
		Elif ((sApp == csForge90) && (iCurFocus == ciRecAdvancedTabCID) && (sCurTab == scAdvancedTab)) then
			; Sound Forge 9 and focus is on the Advanced tab
			let hTmp = FindDescendantWindow(GetParent(hCurFocus),ciPreRecordCID)
		ElIf (IsRecordSpecialButton (GetFocus ())) then
			{Tab}
			Pause ()
			HandleRecButtons ()
			return
		EndIf
		if (sApp == csForge100) then
			; Sound Forge 10, send a Tab and return
			PerformScript Tab ()
			return
		EndIf
		; Give focus to the next non disabled control
		let hTmp = FindNextNonDisabled(hTmp)
		if (not htmp) then
			if (GetParent(hCurFocus) == hRecDlg) then
				let hTmp = GetFirstWindow (hCurFocus)
				let hTmp = FindNextNonDisabled(hTmp)
			else
				let hTmp = FindDescendantWindow(hRecDlg,1)
			EndIf
		EndIf
			setFocus (hTmp)
			return
	EndIf
	PerformScript Tab ()
	if (IsDataWindow()) then
		Pause ()
		let sTmpChannels = gsChannelInfo
		let iNoOfChannels = GetNumberOfChannels ()
		; Loop until the channel info has changed
		if (iNoOfChannels > 1) then
			while (sTmpChannels == gsChannelInfo)
				let gsChannelInfo = GetChannelInfo ()
			EndWhile
		EndIf
		SayActiveChannels ()
	EndIf
EndScript

/******************
 * Description:
 *      Overrided this script to get a Tab order in the record dialog
 * Return:
 *      -
 * Param:
 *      -
 * Note:
 *      For more information read the FSDN help.
 ******************/
Script ShiftTab ()
	var int iCurFocus,
	handle hTmp, handle hCurFocus, handle hRecDlg, string sApp,
	string sCurTab, int iFocusRectLeft, string sTmpChannels, int iNoOfChannels

	if (IsRecordDlg ()) then
		; The record dialog is active
		let iCurFocus = GetControlID (GetFocus ())
		let hCurFocus = GetFocus ()
		let hTmp = hCurFocus
		let sCurTab = GetWindowName (FindDescendantWindow(GetParent(GetFocus ()),4113))
		let hRecDlg = GetRealWindow (GetFocus ())
		let sApp = GetAppFileName ()
		If ((sApp == csForge90) && (iCurFocus == ciMonitorCID)) then
			SetFocus (FindDescendantWindow(hRecDlg,ciRecMetersTabCID))
			return
		ElIf ((sApp == csForge90) && (iCurFocus == ciNewCID)) then
			let hTmp = FindDescendantWindow (hRecDlg,ciWindowCID)
			SetFocus (htmp)
			return
		ElIf ((sApp == csForge90) && (iCurFocus == ciSf90CloseCID)) then
			; Focus is on the Close button
			if (sCurTab == scAdvancedTab) then
				let hTmp = FindDescendantWindow (hRecDlg,ciCalibrateCID)
				SetFocus (hTmp)
				return
			Elif (sCurTab == scMetersTab) then
				let hTmp = FindDescendantWindow (hRecDlg,ciResetCID)
				SetFocus (hTmp)
				return
			EndIf
			ElIf (IsRecordSpecialButton (GetFocus())) then
				{Shift+Tab}
				Pause ()
				HandleRecButtons ()
				return
		EndIf
		if (sApp == csForge100) then
			; Sound Forge 10, just send Shift+Tab and return
			{Shift+Tab}
			return
		EndIf
		let hTmp = FindPrevNonDisabled(hTmp)
		if (not hTmp) then
			let hTmp = FindDescendantWindow (GetParent(GetParent(hCurFocus)),4113)
		EndIf
		setFocus (hTmp)
		return
	endIf
	PerformScript ShiftTab ()
	if (IsDataWindow()) then
		Pause ()
		let sTmpChannels = gsChannelInfo
		let iNoOfChannels = GetNumberOfChannels ()
		if (iNoOfChannels > 1) then
			; Loop until the channel info has changed
			while (sTmpChannels == gsChannelInfo)
				let gsChannelInfo = GetChannelInfo ()
			EndWhile
		EndIf
		SayActiveChannels ()
	EndIf
EndScript
/******************
 * Description:
 *      Overrided this script to make the enter key activate the Scan Levels button in the Noise Reduction dialog
 * Return:
 *      -
 * Param:
 *      -
 * Note:
 *      For more information read the FSDN help.
 ******************/
Script Enter ()
	var handle hWnd
	let hWnd = GetFocus ()
	if (GetControlID(hWnd) == ciScanLevelCID) then
		; Special handling for the Scan Levels button in the Noise Reduction dialog
		SaveCursor ()
		RouteJawsToPc ()
		JawsCursor ()
		SaveCursor ()
		LeftMouseButton ()
		RestoreCursor ()
		RestoreCursor ()
		return
	ElIf (IsFilePropertiesWindow ()) then
		; Focus is in the file properties list in SF 10
		{RightArrow}
		{RightArrow}
		{Enter}
		return
	EndIf
	PerformScript Enter ()
EndScript

; Start the timer
Void Function StartTimer ()
	let giTimer = ScheduleFunction ("Timer", TIMERFREQ)
EndFunction

; This function is called frequently
Void Function Timer ()
	if (IsDataWindow ()) then
		; Only update info if we are in the data window
		let gsTimers = GetTimers()
		if (IsDataSelected ()) then
			if ((GetSelectionStart () != gsSelectionStart) || (GetSelectionEnd () != gsSelectionEnd)) then
				SayString(msgSelectedFrom)
				SayString (ParseTimeString(GetSelectionStart()))
				SayString (msgSelectedTo)
				SayString (ParseTimeString(GetSelectionEnd ()))
			EndIf
			let gsSelectionStart = GetSelectionStart ()
			let gsSelectionEnd = GetSelectionEnd ()
		else
			let gsSelectionStart = ""
			let gsSelectionEnd = ""
		EndIf
	EndIf
	If (IsDataWindow() || IsRecordDlg()) then
		let gsPeakMeters = GetPeakMeters()
		if (giClipping) then
		; Clipping alarm is on
			if (PeakClipping(gsPeakMeters)) then
				; Clipping has occured
				Beep ()
			EndIf
		EndIf
	EndIf
	if (IsRecordDlg ()) then
saystring("hej")
		if (IsRecording ()) then
			if (not gbRecording) then
				; Recording is just activated
				SayMessage (OT_JAWS_MESSAGE,msgRecording,msgRecording)
				let gbRecording = TRUE
				; Update the label on the record button
				if (gsButtonValue == msgRecord) then
					let gsButtonValue = msgStop
				EndIf
			EndIf
		else
			let gbRecording = FALSE
			; Update the label on the stop button
			if (gsButtonValue == msgStop) then
				let gsButtonValue = msgRecord
			EndIf
		EndIf
	EndIf
	; Schedule a new call to the timer
	let giTimer = ScheduleFunction ("Timer", TIMERFREQ)
EndFunction

; Loops through all windows on a level to find the next non-disabled window
HANDLE Function FindNextNonDisabled (handle hWnd)
	var handle hTmp, int bFound, handle hRes,
	string sClass
	let hTmp = GetNextWindow(hWnd)
	while (hTmp && (not bfound))
		let sClass = GetWindowClass (hTmp)
		if (IsWindowVisible (hTmp) && not IsWindowDisabled (hTmp) && (sClass != "static") && (sClass != "SfKrn_Spinner_Class") && (sClass != "SfButton") && (sClass != "#32770") && (sClass != "Forge9.Class.RecordMeters")) then
			let hRes = hTmp
			let bFound = 1		
		EndIf
		let hTmp = GetNextWindow(hTmp)
	EndWhile
	if (bFound) then
		return hRes
	else
		return 0
	EndIf
EndFunction

; Loops backwards through all windows on a level and returns the first previous non-disabled window
HANDLE Function FindPrevNonDisabled (handle hWnd)
	var handle hTmp, int bFound, handle hRes,
	string sClass
	let hTmp = GetPriorWindow(hWnd)
	while (hTmp && (not bfound))
		let sClass = GetWindowClass (hTmp)
		if (not IsWindowDisabled (hTmp) && not StringContains(sClass,"Static") && not StringContains(sClass,"SfKrn_Spinner_Class") && not StringContains(sClass,"SfKrn_Static_Class")) then
			let hRes = hTmp
			let bFound = 1		
		EndIf
		let hTmp = GetPriorWindow(hTmp)
	EndWhile
	if (bFound) then
		return hRes
	else
		return 0
	EndIf
EndFunction

; Returns TRUE if focus is in the record dialog
Int Function IsRecordDlg ()
	var handle hWnd,
	string sWinName
	let hWnd = GetTopLevelWindow (GetFocus())
	let sWinName = GetWindowName (hWnd)
	return (StringContains (	sWinName,scRecordDlg) && StringContains(scRecordDlg,sWinName))
EndFunction

; Returns TRUE if hWnd is the window holding the special buttons in the record dialog
int Function IsRecordSpecialButton (handle hWnd)
	return ((GetWindowClass(hwnd) == csForgeButtonsClass) && (GetControlID(hWnd) == ciRecButtonsCID))
EndFunction

int Function IsRecordStatus (handle hWnd)
	return ((GetWindowClass(hwnd) == csForgeButtonsClass) && (GetControlID(hWnd) == ciRecStatusCID))
EndFunction


int Function IsSF11Recording ()
	var handle hWnd, int iRet
	let hWnd = FindWindowWithClassAndId (GetAppMainWindow(GetFocus()), "SfKrn_TabControl_Class", 48)
	SaveCursor()
	InvisibleCursor()
	SaveCursor ()
	MoveToWindow (hWnd)
	let iRet = GetColorText ()
	RestoreCursor()
	RestoreCursor ()
	return iRet
EndFunction

; Returns TRUE if focus is in the data window
Int Function IsDataWindow ()
	var string sClass,
	string sAppName
	let sAppName = GetAppFileName ()
	let sClass = GetWindowClass(GetFocus ())
	if (not MenusActive () && (sClass == csDataWndClass)) then
		return TRUE
	Else
		return FALSE
	EndIf
EndFunction

; Returns TRUE if focus is in the file properties window
int function IsFilePropertiesWindow ()
	var string sClass, string sRealName
	let sClass = GetWindowClass (getFocus())
	let sRealName = GetWindowName(GetRealWindow(GetFocus()))
	return ((sClass == csSFSpread) && (sRealName == scFilePropertiesTitle))
EndFunction

; Adjusts the size of the peak meter window so all channels could be shown
Script AdjustPeakMetersWindow ()
	var handle Hwnd,
	int x, int y, int iWidth, int iCurWidth, int iCurHeight, int iNoChannels, int wParam
	let iNoChannels = GetNumberOfChannels ()
	if (not iNoChannels) then
		DebugMessage("0 channels",0)
		return
	EndIf
	let hWnd = GetPeakWindow ()
	if (not hWnd) then
		DebugMessage (msgPeakWindowNotFound,0)
		return
	EndIf
	let x = GetWindowLeft (hWnd)
	let y = GetWindowTop (hWnd) + ((GetWindowBottom (hWnd) - GetWindowTop(hWnd))/2)
	let iCurWidth = GetWindowRight (hWnd) - GetWindowLeft (hWnd)
	let iCurHeight = GetWindowBottom(hWnd) - GetWindowTop(hWnd)
	let iWidth = ((iNoChannels + iNOChannels % 2)* 32)
	if (iWidth < 10) then
		SayString(msgWrongWidth)
		return
	EndIf
	SaveCursor ()
	JawsCursor ()
	SaveCursor ()
	MoveTo (x-2, y)
	LeftMouseButtonLock ()
	MoveTo (x-5, y)
	MoveTo (x- (iWidth - iCurWidth), y)
	LeftMouseButtonLock ()
	RestoreCursor ()
	RestoreCursor ()
EndScript

; Do initialization
Void Function Setup ()
	var handle hWnd
	SayMessage(OT_JAWS_MESSAGE,msgInitialize,msgInitialize)

	; Maximize the application window
	; To get proper function in W7 we need to maxmize, restore and then maximize again
	SendMessage(GetAppMainWindow(GetFocus ()),WM_SYSCOMMAND, SC_MAXIMIZE,0)
	SendMessage(GetAppMainWindow(GetFocus ()),WM_SYSCOMMAND, SC_RESTORE,0)
	SendMessage(GetAppMainWindow(GetFocus ()),WM_SYSCOMMAND, SC_MAXIMIZE,0)
	; Organize the peak meters vertically
	SendMessage(GetAppMainWindow(GetFocus()), WM_COMMAND,MENU_VERTICAL,0)
EndFunction

; Used to present a list of tabs in a multi-page dialog
Script SelectTab ()
	var handle hWnd, string sName, int iIndex, string sTabList, string sText,
	string sChunk, string sOldChunk, int bFound
	let hWnd = FindDescendantWindow(GetRealWindow(GetFocus ()),201)
	if (not hWnd) then
		SayMessage (OT_JAWS_MESSAGE,msgNoTabsFound, msgNoTabsFound)
		return
	EndIf
	SaveCursor ()
	JawsCursor ()
	SaveCursor ()
	MoveToWindow (hWnd)
	SetRestriction (RestrictWindow)
	let sChunk = GetChunk ()
	while ((sChunk != sOldChunk) && not bFound)
		let sTabList = sTabList + sChunk + "\007"
		let sOldChunk = sChunk
		NextChunk ()
		let sChunk = GetChunk ()
	EndWhile		
	RestoreCursor ()
	RestoreCursor ()
	let iIndex = DlgSelectItemInList (sTabList, msgSelectTab, FALSE, 1)
	if (iIndex) then
		; The user didn't select cancel
		SaveCursor ()
		JawsCursor ()
		SaveCursor ()
		; Find the desired tab
		if (FindString (hWnd, StringSegment(sTabList,"\007",iIndex), S_BOTTOM, S_UNRESTRICTED)) then
			LeftMouseButton ()
		else
			SayMessage (OT_JAWS_MESSAGE, msgTextNotFound, msgTextNotFound)
		EndIf
		RestoreCursor ()
		RestoreCursor ()
	EndIf
EndScript

; Toggle speech on/off
Script ToggleSpeech ()
	var int bSpeechOff
	let bSpeechOff = IsSpeechOff ()
	if (not bSpeechOff) then
		SayMessage (OT_JAWS_MESSAGE,msgSpeechOff)
		Delay (10,1)
		MuteSynthesizerToggle (FALSE)
	Else
		MuteSynthesizerToggle (FALSE)
		SayMessage (OT_JAWS_MESSAGE,msgSpeechOn)
	EndIf
EndScript

Script SwitchPanes ()
	{Ctrl+F6}
	Pause ()
	if (IsDataWindow ()) then
		SayMessage (OT_JAWS_MESSAGE,GetWindowName(GetFocus()),GetWindowName(GetFocus ()))
	EndIf
EndScript

Int Function CheckRequirements()
	Var
	Int bReturn, string sVersion
    
 Let bReturn = TRUE
  
	If( GetJfwVersion () < 1000000 ) Then
		SayString(msgReqJawsVersion)
		let bReturn = FALSE
	EndIf

 If( (ScreenGetHeight() < 1024 ) || (ScreenGetWidth() < 1280 ) ) Then
   SayString(msgReqResolution)
		let bReturn = FALSE
 EndIf

	let sVersion = GetVersionInfoString (GetAppFilePath(),"FileVersion")
	if (not StringContains (msgSupportedSFVersion,sVersion)) then
		SayString(msgReqSFVersion)
		let bReturn = FALSE
 EndIf

	return bReturn
EndFunction

; Returns TRUE if recording
Int Function IsRecording ()
	var handle hWnd, string sText
	let hWnd = FindDescendantWindow(GetRealWindow(GetFocus()),ciRecordingTextCID)
	let sText = GetWindowText(hWnd,0)
 	return StringContains(sText,scRecording)
EndFunction

Script HotKeysInRecordDlg ()
	var string sKey, handle hWnd, int iCID, string sAppName
	let sAppName = GetAppFileName ()
	let sKey = GetCurrentScriptKeyName ()
	if (isRecordDlg ()) then
		if (sKey == "Alt+T") then
			let iCID = ciResetCID
		ElIf (sKey == "Alt+C") then		
			if (sAppName == csForge90) then
				let iCID = ciSF90CloseCID
			else
				let iCID = ciSF10CloseCID
			EndIf
		EndIf
		let hWnd = FindDescendantWindow (GetRealWindow(GetFocus()),iCID)
		SetFocus (hWnd)
		{Space}
	Else
		TypeKey(sKey)
	EndIf
EndScript

Script PressRecord ()
	var handle hWnd
	if (IsRecordDlg ()) then
		let ghWnd = FindWindow(GetRealWindow(GetFocus()),csForgeButtonsClass,"")
		{Alt+r}
		ScheduleFunction("SetBackFocus",5)
	else
		{Alt+r}
	EndIf	
EndScript


void function SetBackFocus ()
	SetFocus (ghWnd)
EndFunction

Function ReadIniFile ()
	var string sRes, int iX, int iY
	let iX = ScreenGetWidth ()
	let iY = ScreenGetHeight ()
	let giDebug = IniReadInteger ("forge", "debug", 0, "forge.ini")
	let giFlashMessageDuration = IniReadInteger ("forge", "FlashMessageDuration", 0, "forge.ini")
	let gsRecButtonPixels = IniReadString ("forge", "RecButtonPixels"+IntToString(iX)+"x"+IntToString(iY), "", "forge.ini")
	let gsCustomControlBase = IniReadString ("forge", "CustomControlBase", "", "forge.ini")
EndFunction

; Toggle automatic looping of a short region to the right of the cursor
Script ToggleLoopRight ()
	var int iCur, int iTot
	if (giTrim == 1) then
		let giTrim = 0
		UnScheduleFunction(giTrimID)
		SayString(MsgLoopingOff)
		{Space}
		return
	ElIf (not giTrim) then
		let iCur = StringToInt(TimeToMSec(GetSelectionStart ()))
		let iTot = StringToInt(TimeToMSec(GetTotalTime ()))
		if ((iTot - iCur) < 1000) then
			SayString(msgTooCloseToEndOfFile)
			return
		EndIf
		let giTrim = TRUE
		sayString(msgLoopRightOn)
		PerformScript LoopRight ()
	EndIf
EndScript

; Toggle automatic looping of a short region to the leftof the cursor
Script ToggleLoopLeft ()
	var string sTime, int iTime
	if (giTrim == 2) then
		let giTrim = FALSE
		UnScheduleFunction(giTrimID)
		SayString(msgLoopingOff)
		{Space}
		Pause ()
		Pause () Pause ()
		let gsTimers = GetTimers ()
		Pause ()
		let iTime = StringToInt(TimeToMsec(GetSelectionStart()))
		let sTime = IntToString((iTime + 1000) / 1000) + "," + IntToString((iTime + 1000) % 1000)
		{Ctrl+g}
		TypeString (sTime)
		WaitForWindow ("edit",1000)
		{Enter}
		return
	ElIf (not giTrim) then
		let gsTimers = GetTimers()
		let sTime = GetSelectionStart()
		let iTime = StringToInt(TimeToMsec(sTime))
		let iTime = iTime - 1000
		if (iTime < 0) then
			SayString (msgTooCloseToBeginningOfFile)
			return
		EndIf
		let giTrim = 2
		sayString(msgLoopLeftOn)
		{Ctrl+g}
		WaitForWindow ("edit",1000)
		TypeString(IntToString(iTime / 1000) + "," + IntToString(iTime % 1000))
		{Enter}
		Pause ()
		PerformScript LoopLeft ()
	EndIf
EndScript

Script LoopLeft ()
	{Space}
	let giTrimID = ScheduleFunction("StopLeft",10)
EndScript

Script LoopRight ()
	{Space}
	let giTrimID = ScheduleFunction("StopRight",10)
EndScript

Void Function StopRight ()
	{Space}
	PerformScript LoopRight()
EndFunction

Void Function StopLeft ()
	{Space}
	PerformScript LoopLeft ()
EndFunction

Int Function IsMovementKey (string sKey)
	if (StringContains (sKey,"Arrow")) then
		return TRUE
	else 
		return FALSE
	EndIf
EndFunction

Script MarkIn (string sScript, string sFrame)
	SayMessage (OT_KEYBOARD, GetCurrentScriptKeyName (), GetCurrentScriptKeyName ())
	if (giTrim == 1) then
		PerformScript ToggleLoopRight ()
		pause ()
	Elif (giTrim == 2) then
		PerformScript ToggleLoopLeft ()
		Pause ()
	EndIf
	{i}
EndScript

Script MarkOut (string sScript, string sFrame)
	SayMessage (OT_KEYBOARD, GetCurrentScriptKeyName (), GetCurrentScriptKeyName ())
	if (giTrim == 1) then
		PerformScript ToggleLoopRight ()
		pause ()
	Elif (giTrim == 2) then
		PerformScript ToggleLoopLeft ()
		Pause ()
	EndIf
	{o}
EndScript

Int Function IsLoopStartKey (string sKey)
if ((GetScriptKeyName ("ToggleLoopRight") == sKey) || (GetScriptKeyName("ToggleLoopLeft") == sKey)) then
		return TRUE
	EndIf
	return FALSE
EndFunction

;int function CheckGraphic(int left, int top, int right, int bottom)
;	SayString(GetTextInRect (left, top, right, bottom, 0, 0, 0, 0, 0))
;	return TRUE
;EndFunction

int Function CheckLicense ()
	if (not gbChecked) then
		SayString(msgIncorrectLicense)
		return FALSE
	EndIf
	return TRUE
EndFunction


int Function LoadLicense ()
	var string sFile, int iLicenseCode, string sApp, string sVersion, int iIndex, int bFound,
	int iMajor, int iMinor, int iUpdate, int iBuild,
	string sSerial, string sCurLicenseCode, int iCurLicenseCode

	; Get the Sound Forge version info
	GetFixedProductVersion  (GetAppFilePath(), iMajor, iMinor, iUpdate, iBuild)	
	let sVersion = IntToString(iMajor)

	; Pad the version number with 0 up to two characters
	let sVersion = StringLeft (PADSTRING, 2 - StringLength (sVersion)) + " " + sVersion
	let sVersion = StringStripAllBlanks (sVersion)

	let iIndex = 1

	; Loop to find the index into the APPLIST array
	while (StringSegment (VERLIST,"|",iIndex) && not bFound)
		if (StringSegment (VERLIST,"|",iIndex) == sVersion) then
			let sApp = StringSegment (APPLIST,"\007",iIndex)
			let bFound = TRUE
		EndIf
		let iIndex = iIndex + 1
	EndWhile

	; Construct the license file name
	let sFile = LIC_FILE_PREFIX + IntToString (GetJfwSerialNumber ()) + LIC_FILE_SUFIX
	; Read the license code
	let iLicenseCode = IniReadInteger (sApp, LIC_KEY, 0, sFile)

	if (not iLicenseCode) then
		SayMessage (OT_JAWS_MESSAGE, msgLicenseFileError, msgLicenseFileError)
		return FALSE
	EndIf

	; Compare the license code from the file with the real license
	; Get JAWS serial number
	let sSerial = IntToString (GetJFWSerialNumber ())
	; Append the serial number and SF version
	let sCurLicenseCode = sSerial + " " + sVersion
	let sCurLicenseCode = stringStripAllBlanks (sCurLicenseCode)
	; convert to an int and shift left 1 step
	let iCurLicenseCode = StringToInt (sCurLicenseCode) << 1
	return (iLicenseCode == iCurLicenseCode)
EndFunction

; Moves to location sTime
Function MoveToLocation (string sTime)
	{Ctrl+g}
	WaitForWindow ("edit",1000)
	TypeString(sTime)
	{Enter}
EndFunction

Script ListGraphicIcons ()
	var handle hWnd, int iIndex, string sSelected

	; Get the handle of the window containing the graphical buttons
	let hWnd = FindDescendantWindow (GetRealWindow(GetFocus()),ciPluginGraphicalButtons1 )
	if (not hWnd) then
		SayMessage (OT_JAWS_MESSAGE,msgCommandNotValid,msgCommandNotValid)
		return
	EndIf

	let gsGraphicsList = ""

	; Enumerate all graphical buttons
	GraphicsEnumerate (hWnd, "GraphEnum")

	let hWnd = FindDescendantWindow  (GetRealWindow(GetFocus()),ciPluginGraphicalButtons2 )
	if (hWnd) then
		GraphicsEnumerate (hWnd, "GraphEnum")
	EndIf

	; Show the list of buttons
	let iIndex = DlgSelectItemInList (gsGraphicsList, "List of graphics", FALSE, 1)
	if (iIndex) then
		; The user didn't select Cancel
		let sSelected = StringSegment(gsGraphicsList,"\007",iIndex)
		SaveCursor ()
		JawsCursor ()
		SaveCursor ()

		; Move to the graphical button
		if (FindGraphic (hWnd, sSelected, S_TOP, S_UNRESTRICTED)) then
			LeftMouseButton ()
		EndIf
		RestoreCursor ()
		RestoreCursor ()
	EndIf
EndScript

Function GraphEnum (int nLeft, int nTop, int nRight, int nBottom)
	var string sGraphicsName
	; Get the text asociated with the icon
	let sGraphicsName = GetTextInRect (nLeft, nTop, nRight, nBottom, 0, 0, 0, 0, 0)
	; Add the icon to the graphics list
	let gsGraphicsList = gsGraphicsList + sGraphicsName + "\007"
	return True
EndFunction

Script ListPluginChainerTabs ()
	var handle hWnd, string sText, int iIndex, string sTabs

	; Find the handle of the tab control window
	let hWnd = FindWindow (GetRealWindow(GetFocus()), csPluginChainerTabs)

	if (not hWnd) then
		SayMessage (OT_JAWS_MESSAGE,msgCommandNotValid,msgCommandNotValid)
		return
	EndIf

	SaveCursor ()
	BrailleCursor ()
	SaveCursor ()
	MoveToWindow(hWnd)
	SetRestriction (RestrictWindow)

	; Get the text of the tabs
	let sTabs = GetLine ()
	RestoreCursor ()
	RestoreCursor ()

	; The tabs are separated by 3 spaces, replace those with the list delimiter
	let sTabs = StringReplaceSubStrings (sTabs, "   ", "\007")

	; Present the list
	let iIndex = DlgSelectItemInList (sTabs, "Select plugin", 0, 1)

	; Get the text of the selected tab
	let sText = StringSegment(sTabs,"\007",iIndex)

	if (iIndex) then
		; Chop the checkbox
		let sText = StringChopLeft(sText,3)
		SaveCursor ()
		JawsCursor ()
		if (FindString(hWnd,sText,S_TOP,S_UNRESTRICTED)) then
			RightMouseButton ()
		EndIf
		RestoreCursor ()			
		RestoreCursor ()
	EndIf
EndScript









