Include "Forge.jsh"
Include "forge.jsm"
Include "hjconst.jsh"
Include "hjglobal.jsh"
Use "ForgeLib.jsb"
use "forgemarker.jsb"
use "forgetrack.jsb"
; Secondary routing button
Script FHPRoutingAUX ()
	var int iSel, string sTime, int iMarker, int nCell, string sMarkers,
	int iTrack, string sTracks,
	string sTrackNO, string sTrackStart, string sTrackEnd, string sTrack
	let nCell = GetLastBrailleRoutingKey ()
	if ((GetBrailleDisplayName () == "ALVA_BC6") && (BrailleGetCellCount () == 80)) then
	; There is an offset of 228 when using Alva BC680	
			let nCell = nCell - 228
	EndIf
	if (giMode == MARKERS) then
		; Markers mode is active
		let sMarkers = StringSegment (gsBrailleMarkersLine,"|",nCell)		
		if (StringSegmentCount (sMarkers,"#") >= 1) then
			; Secondary routing button is pressed on a marker or region
			SayMessage (OT_JAWS_MESSAGE,msgPerformActionOnMarker,msgPerformActionOnMarker)
			if (StringSegmentCount(sMarkers,"#") == 1) then
				; Only one marker or region is mapped to this cell
				let iMarker = StringToInt(StringSegment(sMarkers,"#",1))
			ElIf (StringSegmentCount(sMArkers,"#") > 1) then
				; More than one marker is mapped to this cell, show a menu
				let iMarker = SelectMarkerFromMenu(sMarkers)
			EndIf
			if (iMarker > 0) then
				if (not IsRegion(iMarker)) then
					; This was a marker
					ExecuteMarkerAction(iMarker)
				Else
					; This was a region
					ExecuteRegionAction (iMarker)
				EndIf
			EndIf
			return
		EndIf
		SayMessage(OT_JAWS_MESSAGE,msgNoMarkerOrRegion,msgNoMarkerOrRegion)
	ElIf (giMode == TRACKS) then
		; Track mode is active
		let sTracks = StringSegment(gsBrailleTracksLine,"|",nCell)		
		if (StringSegmentCount(sTracks,"#") >= 1) then
			; Secondary routing button is pressed on a track marker
;		SayMessage(OT_JAWS_MESSAGE,msgPerformActionOnMarker,msgPerformActionOnMarker)
			if (StringSegmentCount(sTracks,"#") == 1) then
				; One track
				let iTrack = StringToInt(StringSegment(sTracks,"#",1))
		else
				; More than one track marker
;				let iTrack = SelectMarkerFromMenu(sMarkers)
			EndIf
			if (iTrack > 0) then
				let giIndex = iTrack
				ShowInfoMessage ()
			EndIf
			return
		EndIf
;		SayMessage(OT_JAWS_MESSAGE,msgNoMarkerOrRegion,msgNoMarkerOrRegion)
	EndIf
EndScript

; Customized version of BrailleRouting
; Needed to make it possible to click CR to reset a peak meter and to move in markers mode
Script BrailleRouting()
	var string sChannels, int nCell, int x, int y, int iMarker, int iTrack,
	string sMarkers, string sTime, string sTracks, int iIndex, int iNoOfChannels, string sColor, string sActiveChannels,
	handle hWnd
	let nCell = GetLastBrailleRoutingKey ()
	let x = GetBrailleCellColumn (nCell)
	let y = GetBrailleCellRow (nCell)
	if (IsDataWindow ()) then
	; Focus is in the data window
		; Close all module windows if any open
		CloseListWindows ()
		if (giMode == MARKERS) then
			; markers mode is active
			; Get markers for the cell corresponding to the CR button
			let sMarkers = StringSegment (gsBrailleMarkersLine,"|",nCell)		
			if (StringSegmentCount (sMarkers,"#") == 1) then
				; Only one marker or region
				let iMarker = StringToInt (StringSegment(sMarkers,"#",1))
			ElIf (StringSegmentCount(sMArkers,"#") > 1) then
				; More than one marker or region, show a menu
				let iMarker = SelectMarkerFromMenu (sMarkers)
			EndIf
			if (iMarker > 0) then
			; One or more marker at this cell
				MoveToMarkerByIndex (iMarker)
				if (IsRegion (iMarker)) then
					SayMessage (OT_JAWS_MESSAGE, msgMoveToRegion+GetMarkerName(GetMarkerByIndex(gsMarkers,iMarker)),"")
				else
					SayMessage (OT_JAWS_MESSAGE, msgMoveToMarker+StringSegment(StringSegment(gsMarkers,"|",iMarker),"#",2), "")
				EndIf
				let giIndex = iMarker
				ShowInfoMessage ()
			else
			; No marker or region
				let sTime = GetLocationFromBrailleCell(nCell)
				MoveToLocation(sTime)
				SayMessage (OT_JAWS_MESSAGE,msgMoveToLocation+ ParseTimeString(sTime),"")
			EndIf
		ElIf (giMode == TRACKS) then
			; Tracks mode is active
			; Get tracks for the cell corresponding to the CR button
			let sTracks = StringSegment(gsBrailleTracksLine,"|",nCell)		
			if (StringSegmentCount(sTracks,"#") == 1) then
				; Only one marker or region
				let iTrack = StringToInt(StringSegment(sTracks,"#",1))
			ElIf (StringSegmentCount(sTracks,"#") > 1) then
				; More than one track
				let iTrack = SelectTrackFromMenu(sTracks)
			EndIf
			if (iTrack > 0) then
				MoveToTrackByIndex (iTrack)
				SayMessage (OT_JAWS_MESSAGE, msgMoveToTrack+StringSegment(StringSegment(gsTracks,"|",iTrack),"#",1)," ")
				let giIndex = iTrack
				ShowInfoMessage()
			Else
				let sTime = GetLocationFromBrailleCell (nCell)
				SayMessage (OT_JAWS_MESSAGE,msgMoveToLocation+ ParseTimeString(sTime),"")
				MoveToLocation(sTime)
			EndIf
		ElIf (giMode == TIME) then
			; Time mode is active
			let sTime = GetLocationFromBrailleCell (nCell)
			SayMessage (OT_JAWS_MESSAGE,msgMoveToLocation+ ParseTimeString(sTime),"")
			MoveToLocation(sTime)
		ElIf ((giMode == CHANNELS) && (x > 1)) then
			; Channel mode and valid channel is clicked
			; Get array of channels
			let sChannels = GetChannelInfo ()
			let iNoOfChannels = GetNumberOfChannels ()
			let sActiveChannels = GetActiveChannels(sChannels)
			if (iNoOfChannels == StringSegmentCount(sActiveChannels,",")) then
				; All channels activated, user deactivates
				SayMessage (OT_JAWS_MESSAGE,FormatString (msgChannelDeActivated, IntToString(x-1)),"")
				let sColor = StringSegment(csUnSelChannelColors,"|",x-1)
			Elif (y == 1) then
				; User wants to activate a channel
				SayMessage (OT_JAWS_MESSAGE,FormatString (msgChannelActivated, IntToString(x-1)),"")
				let sColor = StringSegment(csUnSelChannelColors,"|",x-1)
			else
				; User wants to deactivate a channel
				let sColor = StringSegment(csSelChannelColors,"|",x-1)
				SayMessage (OT_JAWS_MESSAGE,FormatString (msgChannelDeActivated, IntToString(x-1)),"")
			EndIf
			SaveCursor ()
			JawsCursor ()
			SaveCursor ()

			RouteJawsToPc ()
			SetRestriction(RestrictWindow)
			if(FindColors (IgnoreColor,StringToInt(sColor), S_BOTTOM, TRUE) && (GetWord() == IntToString(x-1))) then
				ControlLeftMouseClick ()
			Elif(FindColors (IgnoreColor,StringToInt(sColor), S_PRIOR, TRUE) && (GetWord() == IntToString(x-1))) then
				ControlLeftMouseClick ()
			EndIf
			RestoreCursor ()
			RestoreCursor ()
			Pause ()
			; Update channel info
			let gsChannelInfo = GetChannelInfo ()
		ElIf ((giMode == PEAK) && (x > 1)) then
			; Peaks mode is active
			; Click to clear a peak meter
			ClearPeakMeter(x - 1)
		EndIf
	else
		; Default handling
		PerformScript BrailleRouting()
	EndIf
EndScript


String Function GetLocationFromBrailleCell (int nCell)
	var int iMSecPerCell, int iLoc,
	string sHours, string sMinutes, string sSeconds, string sMSec, string sTime
	let sTime = GetSelectionStart ()
	let iMSecPerCell = (StringToInt (TimeToMSec(GetTotalTime ()))) / BrailleGetCellCount ()
	; Location to move to in msec
	let iLoc = (nCell - 1) * iMSecPerCell
	; Calculate the time in hours, minutes, seconds and mseconds
	let sHours  = IntToString(iLoc / (3600 * 1000))
	let sMinutes = IntToString((iLoc % (3600 * 1000)) / (60 * 1000))
	let sSeconds = IntToString(((iLoc % (3600 * 1000)) % (60 * 1000)) / 1000)
	let sMSec = IntToString(((iLoc % (3600 * 1000)) % (60 * 1000)) % 1000)
	let sMSec = StringRight(" 000" + sMSec,3)
	if (StringContains(sTime,",")) then
	; , as decimal character
		let sTime = sHours + ":" + sMinutes + ":" + sSeconds + "," +sMSec
	else
	; . as decimal character
		let sTime = sHours + ":" + sMinutes + ":" + sSeconds + "." +sMSec
	EndIf
	return sTime
EndFunction

; EAB one step up
Script FHPBraillexEL1U ()
	var string sApp
	let sApp = GetAppFileName ()
	if (IsDataWindow ()) then
		if (giMode == TIME) then
			let giMode = PEAK
			SayMessage(OT_JAWS_MESSAGE,msgPeakMode,"")
		ElIf (giMode == PEAK) then
			let giMode = MARKERS
			SayMessage(OT_JAWS_MESSAGE,msgMarkersMode,"")
				UpdateMarkers ()
		ElIf (giMode == MARKERS) then
			if ((sApp == csForge100) || (sApp == csForge110)) then
				let giMode = TRACKS
				SayMessage(OT_JAWS_MESSAGE,msgTRACKSMode,"")
					UpdateTracks ()
			else
				let giMode = CHANNELS
				SayMessage(OT_JAWS_MESSAGE,msgChannelsMode,"")
			EndIf
		ElIf (giMode == TRACKS) then
			let giMode = CHANNELS
			SayMessage(OT_JAWS_MESSAGE,msgChannelsMode,"")
		else
			let giMode = TIME
			SayMessage(OT_JAWS_MESSAGE,msgTimeMode,"")
		EndIf
	else
		FHPBraillexEL1U ()
	EndIf
EndScript

; EAB one step down
Script FHPBraillexEL1D ()
	var string sApp
	let sApp = GetAppFileName ()
	if (IsDataWindow ()) then
		if (giMode == MARKERS) then
			let giMode = PEAK
			SayMessage (OT_JAWS_MESSAGE,msgPeakMode,"")
		ElIf (giMode == PEAK) then
			let giMode = TIME
			SayMessage (OT_JAWS_MESSAGE,msgTimeMode,"")
		ElIf (giMode == TIME) then
			let giMode = CHANNELS
			SayMessage (OT_JAWS_MESSAGE,msgChannelsMode,"")
		ElIf (giMode == CHANNELS) then
			if ((sApp == csForge100) || (sApp == csForge110)) then
				let giMode = TRACKS
				SayMessage (OT_JAWS_MESSAGE,msgTracksMode,"")
				UpdateTracks ()
			else
				let giMode = MARKERS
				SayMessage (OT_JAWS_MESSAGE,msgMarkersMode,"")
				UpdateMarkers ()
			EndIf
		else
			let giMode = MARKERS
			SayMessage (OT_JAWS_MESSAGE,msgMarkersMode,"")
		EndIf
	else
		FHPBraillexEL1D ()
	EndIf
EndScript

; ***Functions to build the Braille output***
int function BrailleAddObjectRecStatus(int nSubtypeCode)
	var handle hWnd, string sText, int iRecStatus
	if (IsRecordDlg () && IsRecording ()) then
		let hWnd = FindDescendantWindow (GetRealWindow(GetFocus()),ciRecordingTime)
		let sText = GetWindowName (hWnd)
		if (sText) then
			BrailleAddString(msgRecording + " " + sText,0,0,0)
			return TRUE
		EndIf
	ElIf (IsDataWindow ()) then
			let iRecStatus = IsSF11Recording ()
			if (iRecStatus == 128) then
				BrailleAddString(msgRecording,0,0,0)
				return TRUE
			ElIf (iRecStatus == 32768) then
				BrailleAddString(msgArmed,0,0,0)
				return TRUE
		EndIf
	EndIf
	return FALSE
EndFunction

int function BrailleAddObjectTotalRecTime(int nSubtypeCode)
	var handle hWnd, string sText
	let hWnd = FindDescendantWindow (GetRealWindow(GetFocus()),ciRecStartCID)
	let sText = GetWindowText(hWnd,0)
	if (sText) then
		BrailleAddString(sText,0,0,0)
		return TRUE
	else
		return FALSE
	EndIf
EndFunction

int function BrailleAddObjectName(int nSubtypeCode)
	var string sLabel
	if ((nSubtypeCode == WT_BUTTON) && IsRecordSpecialButton (GetFocus ())) then
		; Buttons in the record dialog
		BrailleAddString (gsButtonValue,0,0,0)
		return TRUE
	ElIf (nSubTypeCode == WT_DIALOG) then
		let sLabel = GetWindowName (GetRealWindow(GetFocus ()	))
		; Chop the file name from the dialog title
		let sLabel = StringSegment (sLabel, " ",1)
		BrailleAddString (sLabel, 0, 0, 0)
		return TRUE
	ElIf (IsSFUpDownSlider(GetFocus ())) then
		BrailleAddString(GetSFUpDownSliderLabel(GetFocus()),0,0,0)
		return TRUE
	EndIf
	return FALSE
EndFunction

int function BrailleAddObjectDlgText (int nSubtypeCode)
	if (IsRecordDlg()) then
		return TRUE
	else
		return FALSE
	EndIf
EndFunction

; Show the selection start time (the current time) on the Braille display
Int Function BrailleAddObjectSelectionStart (int nSubtypeCode)
	var string sSelectionStart
	let sSelectionStart = GetSelectionStart ()
	BrailleAddString (sSelectionStart,0,0,0)
	return 1
EndFunction

; Show the selection length when selecting
Int Function BrailleAddObjectSelectionLength (int nSubtypeCode)
	var string sSelectionLength
	let sSelectionLength = GetSelectionLength ()
	BrailleAddString(sSelectionLength,0,0,0)
	return 1
EndFunction

; Show the selection end time when selecting
Int Function BrailleAddObjectSelectionEnd (int nSubtypeCode)
	var string sSelectionEnd
	let sSelectionEnd = GetSelectionEnd ()
	BrailleAddString(sSelectionEnd,0,0,0)
	return 1
EndFunction

; Show the peak meters
Int Function BrailleAddObjectPeaks (int nSubtypeCode)
	var int iIndex,
	string sTmp,
	int bPeak
	let iIndex = 1
	while (StringSegment (gsPeakMeters,"|",iIndex))
		let sTmp = StringSegment (gsPeakMeters,"|",iIndex)
		let bPeak = StringToInt(StringSegment (sTmp,"#",2))
		if (bPeak) then
			BrailleAddString (IntToString(iIndex)+":"+StringSegment(sTmp,"#",1),iIndex,0,ATTRIB_HIGHLIGHT)
		else
			BrailleAddString (IntToString(iIndex)+":"+StringSegment(sTmp,"#",1),iIndex,0,0)
		EndIf
		let iIndex = iIndex + 1
	EndWhile
	return 1
EndFunction

; Show the total time
Int Function BrailleAddObjectTotal (int nSubtypeCode)
	var string sTotal
	let sTotal = GetTotalTime ()
	BrailleAddString(sTotal,0,0,0)
	return 1
EndFunction

; Show NO of channels
Int Function BrailleAddObjectChannels (int nSubtypeCode)
	var string sChannels, int iNoOfChannels
	let sChannels = gsChannelInfo
	let sChannels = GetActiveChannels (sChannels)
	let iNoOfChannels = GetNumberOfChannels ()
	if (iNoOfChannels == StringSegmentCount (sChannels,",")) then
		BrailleAddString (msgBrlAllChannelsActive,0,0,0)		
	else
		BrailleAddString(sChannels,0,0,0)
	EndIf
	return 1
EndFunction

; Show markers
int function BrailleAddObjectMarkers(int nSubtypeCode)
var string sTmp, string sCur, string sTot, string sLoc, string sLine, string sMarkersList,
	int iCur, int iID, int iTot, int iLoc, int iIndex, int iMSecPerCell, int iCursorPos
;	let gsBrailleMarkersLine = GetBrailleMarkersLine ()
	let iIndex = 1
	let sTot = TimeToMSec(GetTotalTime ())
	let iTot = StringToInt (sTot)
	let iMSecPerCell = (iTot) / BrailleGetCellCount ()
	let sCur = TimeToMsec(GetSelectionStart ())
	let iCur = StringToInt(sCur) 
	let iCursorPos = (iCur/iMSecPerCell) + 1
	if (iCursorPos > BrailleGetCellCount ()) then
		let iCursorPos = BrailleGetCellCount ()
	EndIf
	while (iIndex < (BrailleGetCellCount ()) + 1)
		let sMarkersList = StringSegment(gsBrailleMarkersLine,"|",iIndex)
		if (((iCur/iMSecPerCell) + 1) == iIndex) then
				let sLine = sLine + msgBrlCursor
		Elif (((iCur/iMSecPerCell) == iindex) && (iIndex == BrailleGetCellCount ())) then
				; Special case when cursor is at the end of file
				let sLine = sLine + msgBrlCursor
		Elif (sMarkersList == "") then
			if (msgBrlEmptySymbol != "") then
				let sLine = sLine + msgBrlEmptySymbol
			else
				let sLine = sLine + " "
			EndIf
		else
			if (StringSegmentCount(sMarkersList,"#") == 1) then
				let iID = StringToInt(StringSegment(sMarkersList,"#",1))
				if (StringSegment (StringSegment(gsMarkers,"|",iID),"#",4)) then
					let sLine = sLine + msgBrlRegionSymbol
				Else
					let sLine = sLine + msgBrlMarkerSymbol
				EndIf
			Else
				let sLine = sLine + msgBrlDualMarkerSymbol
			EndIf
		EndIf
		let iIndex = iIndex + 1
	EndWhile
;	let sTmp = StringLeft(sLine,iCursorPos - 1)
;	BrailleAddString(sTmp,0,0,0)
;	let sTmp = StringChopLeft(sLine, iCursorPos - 1)
;	let sTmp = StringChopRight(sTmp,StringLength(sTmp)-1)
;	let sTmp = StringRight(sLine,StringLength(sLine)-iCursorPos-1)
;	BrailleAddString(sTmp,0,0,0)
	BrailleAddString(sLine,0,0,0)
	return TRUE
EndFunction

int function BrailleAddObjectTracks(int nSubtypeCode)
var string sTmp, string sCur, string sTot, string sLoc, string sLine, string sTracksList,
	int iCur, int iID, int iTot, int iLoc, int iIndex, int iMSecPerCell, int iCursorPos
;	let gsBrailleTracksLine = GetBrailleTracksLine ()
	let iIndex = 1
	let sTot = TimeToMSec(GetTotalTime ())
	let iTot = StringToInt (sTot)
	let iMSecPerCell = (iTot) / BrailleGetCellCount ()
	let sCur = TimeToMsec(GetSelectionStart ())
	let iCur = StringToInt(sCur)
	while (iIndex < (BrailleGetCellCount () + 1))
		let sTracksList = StringSegment(gsBrailleTracksLine,"|",iIndex)
		if (((iCur/iMSecPerCell)+1) == iIndex) then
				let sLine = sLine + msgBrlCursor
		Elif (((iCur/iMSecPerCell) == iindex) && (iIndex == BrailleGetCellCount ())) then
				; Special case when cursor is at the end of file
				let sLine = sLine + msgBrlCursor
		Elif (sTracksList == "") then
			if (msgBrlEmptySymbol != "") then
				let sLine = sLine + msgBrlEmptySymbol
			else
				let sLine = sLine + " "
			EndIf
		else
			if (StringSegmentCount(sTracksList,"#") == 1) then
					let sLine = sLine + msgBrlTrackSymbol

;				let iID = StringToInt(StringSegment(sTracksList,"#",1))
;				if (StringSegment(StringSegment(gsTracks,"|",iID),"#",4)) then
;					let sLine = sLine + msgBrlRegionSymbol
;				Else
;					let sLine = sLine + msgBrlMarkerSymbol
;				EndIf
			Else
				let sLine = sLine + msgBrlDualTrackSymbol
			EndIf
		EndIf
		let iIndex = iIndex + 1
	EndWhile
	BrailleAddString(sLine,0,0,0)
	return TRUE
EndFunction

int function BrailleAddObjectTrackName (int nSubTypeCode)
	var string sTrack
	let sTrack = GetTrackByIndex(gsTracks,giIndex)
	BrailleAddString(GetTrackName(sTrack),0,0,0)
	return TRUE
EndFunction

int function BrailleAddObjectTrackStart (int nSubTypeCode)
	var string sTrack
	let sTrack = GetTrackByIndex(gsTracks,giIndex)
	BrailleAddString(GetTrackStart(sTrack),0,0,0)
	return TRUE
EndFunction

int function BrailleAddObjectTrackEnd (int nSubTypeCode)
	var string sTrack
	let sTrack = GetTrackByIndex(gsTracks,giIndex)
	BrailleAddString(GetTrackEnd(sTrack),0,0,0)
	return TRUE
EndFunction

int function BrailleAddObjectValue (int nSubTypeCode)
	if (IsSFUpDownSlider(GetFocus())) then
		BrailleAddString(GetSFUpDownSliderValue(GetFocus()),0,0,0)
		return TRUE
	Else
		return FALSE
	EndIf
EndFunction

int Function BrailleAddObjectMarkerName(int nSubTypeCode)
	var string sMarker
	let sMarker = GetMarkerByIndex(gsMarkers,giIndex)
	BrailleAddString(GetMarkerName(sMarker),0,0,0)
	return TRUE
EndFunction

int Function BrailleAddObjectChannelView(int nSubTypeCode)
	var string sChannel, int iIndex
	let iIndex = 1
	while (StringSegment(gsChannelInfo,"|",iIndex))
		let sChannel = StringSegment(gsChannelInfo,"|",iIndex)
		if (StringSegment(sChannel,"#",2) == "1") then
			BrailleAddString (IntTostring(iIndex)+"(x)",iIndex,1,0)
		else
			BrailleAddString (IntToString(iIndex)+"( )",StringToInt(StringSegment(sChannel,"#",1)),0,0)
		EndIf
		let iIndex = iIndex + 1
	EndWhile
	return TRUE
EndFunction

int function BrailleCallbackObjectIdentify()
	var int iCustomControlBase
	let iCustomControlBase = GetCustomControlBase()
	; Turn off braille handling if no license
	if (not CheckLicense ()) then
		return BrailleCallbackObjectIdentify ()
	EndIf

	if (IsDataWindow ()) then
		if (giMode == TIME) then
			if (not GetSelectionEnd ()) then
				; No data selected
				return iCustomControlBase+1
			else 
					; Data selected
					return iCustomControlBase+2
			EndIf
		ElIf (giMode == Peak) then
			return iCustomControlBase+3
		ElIf (giMode == MARKERS) then
			if (not giInfo) then
				; Show info message about marker
				return iCustomControlBase+4
			else
				return iCustomControlBase+7
			EndIf
		ElIf (giMode == TRACKS) then
			if (not giInfo) then
				; Show info message about track
				return iCustomControlBase+5
			else
				return iCustomControlBase+6
			EndIf
			ElIf (giMode == CHANNELS) then
				return iCustomControlBase+8
		EndIf
	Elif (IsRecordDlg () && IsRecordSpecialButton (GetFocus())) then
		; A special button in the record dialog
		return WT_BUTTON
	Elif (IsRecordDlg () && IsRecordStatus (GetFocus())) then
		return WT_STATIC
	ElIf (IsSFUpDownSlider(GetFocus())) then
		; Sound Forge special updown slider
			return WT_UPDOWNSLIDER		
	Else
		; Do the default
		return BrailleCallbackObjectIdentify ()
	EndIf
EndFunction

Function ShowInfoMessage ()
	if (giFlashMessageDuration) then
		let giInfo = TRUE
		ScheduleFunction("ClearInfoMessage",giFlashMessageDuration)
	EndIf
EndFunction
Function ClearInfoMessage ()
	let giInfo = FALSE
EndFunction

