/*************************************
 * STANDARD INCLUDES                 *
 *************************************/
include "hjconst.jsh"

/*************************************
 * Sound Forge specific includes          *
 *************************************/
include "forge.jsh"
include "forge.jsm"

/******************
 * Description:
 *      Returns the selection start time
 * Return:
 *      The selection start time
 * Param:
 *      -
 * Note:
 *      Reads the selection start from the global array gsTimers
 ******************/
String Function GetSelectionStart ()
	if (gsTimers != "") then
		return StringSegment (gsTimers,"|",1)
	else
		return ""
	EndIf
EndFunction

/******************
 * Description:
 *      Returns the end time of the current selection
 * Return:
 *      The end time
 * Param:
 *      -
 * Note:
 *      Reads the end time from the global array gsTimers
 ******************/
String Function GetSelectionEnd ()
	if (gsTimers != "") then
		return StringSegment(gsTimers,"|",2)
	else
		return ""
	EndIf
EndFunction

/******************
 * Description:
 *      Returns the length of the current selection or ""
 * Return:
 *      The length of the current selection
 * Param:
 *      -
 * Note:
 *      Reads the length from the global array gsTimers
 ******************/
String Function GetSelectionLength ()
	if (gsTimers != "") then
		return StringSegment(gsTimers,"|",3)
	else
		return ""
	EndIf
EndFunction

/******************
 * Description:
 *      Returns the current zoom ratio as a string
 * Return:
 *      The zoom ratio
 * Param:
 *      -
 * Note:
 *      Reads the zoom ratio from the global array gsTimers
 ******************/
String Function GetZoomRatio ()
	if (gsTimers != "") then
		return StringSegment (gsTimers,"|",4)
	Else
		return ""
	EndIf
EndFunction

/******************
 * Description:
 *      Returns the total length of the sound file
 * Return:
 *      The length of the file
 * Param:
 *      -
 * Note:
 *      Reads the total time from the status bar 
 ******************/
String Function GetTotalTime ()
	var string sTotalTime,
	string sTmp,
	int iRestriction,
	int iPos,
	string sStatusLine
	let sStatusLine = GetStatusBar ()
	; Extract the total time,
	if (StringContains(sStatusLine,scProcessing)) then
		let iPos = StringContains(sStatusLine,":")
		let sStatusLine = StringChopLeft(sStatusLine,iPos)
	EndIf
	let iPos = StringContains(sStatusLine,":")
	let sTmp = StringChopLeft (sStatusLine,iPos - 3)
	let sTotalTime = StringLeft (sTmp,12)
	return sTotalTime
EndFunction

/******************
 * Description:
 *      Returns the text from the status bar
 * Return:
 *      The text on the status bar
 * Param:
 *      -
 * Note:
 *      - 
 ******************/
String Function GetStatusBar ()
	var handle hWnd, string sStatusLine
	if (GetAppFileName () == csForge90) then
		let hWnd = FindDescendantWindow(GetAppMainWindow(GetFocus()),ciStatusBarForge90CID)
	Elif (GetAppFileName () == csForge100) then
		let hWnd = FindDescendantWindow(GetAppMainWindow(GetFocus()),ciStatusBarForge10CID)
	Elif (GetAppFileName () == csForge110) then
		let hWnd = FindDescendantWindow(GetAppMainWindow(GetFocus()),ciStatusBarForge11CID)
	EndIf
	if (not hWnd) then
		if (giDebug) then
			DebugMessage(msgStatusbarNotFound,0)
		EndIf
		return ""
	EndIf
	let sStatusLine = GetWindowText(hWnd,0)
	return sStatusLine
EndFunction

/******************
 * Description:
 *      Returns an array with info about all channels
 * Return:
 *      The array of channel info
 * Param:
 *      -
 * Note:
 *      - 
 ******************/
String Function GetChannelInfo ()
	var int x, int y,
	string sResult,
	int iIndex,
	int iNoOfChannels, int iActive
	let iIndex = 1
	let iNoOfChannels = GetNumberOfChannels ()
	; Loop to construct the channel info array
	while (iIndex <= iNoOfChannels) 
		if (IsChannelNActive(iIndex)) then
			let sResult = sResult + IntToString(iIndex) +"#"+"1"+ "|"
			let iActive = TRUE
		else
			let sResult = sResult + IntToString(iIndex) +"#"+ "0" + "|"
		endIf
		let iIndex = iIndex + 1
	EndWhile
	if (not iActive) then
		; This means that all channels are active
		let sResult = ""
		let iIndex = 1

		; Construct an array with all channels active
		while (iIndex <= iNoOfChannels)
			let sResult = sResult + IntToString(iIndex) + "#" + "1" + "|"
			let iIndex = iIndex + 1
		EndWhile
	EndIf

	; Chop the | in the end
	let sResult = StringChopRight(sResult,1)
	return sResult
EndFunction

/******************
 * Description:
 *      Returns an array with the active channels
 * Return:
 *      The array of active channels
 * Param:
 *      An array of channel info
 * Note:
 *      - 
 ******************/
String Function GetActiveChannels (string sChannels)
	var int iIndex, string sResult
	iIndex = 1
	while (StringSegment(sChannels,"|",iIndex))
		if (StringSegment(StringSegment(sChannels,"|",iIndex),"#",2) == "1") then
			; This channel is active
			let sResult = sResult + StringSegment (StringSegment(sChannels,"|",iIndex),"#",1) + ","
		EndIf
		let iIndex = iIndex + 1
	EndWhile
	if (sResult) then
		; Chop the delimitor at the end
		let sResult = StringChopRight(sResult,1)
	EndIf
	return sResult
EndFunction

Int Function IsChannelNActive (int iChannel)
	var string sColors, string sColor, int bRet
	let sColors = csSelChannelColors 
	let sColor = StringSegment(sColors,"|",iChannel)
	SaveCursor ()
	InvisibleCursor ()
	SaveCursor ()
	RouteInvisibleToPc ()
	SetRestriction(RestrictWindow)
	if(FindColors (IgnoreColor,StringToInt(sColor), S_TOP, TRUE) && (GetWord() == IntToString(iChannel))) then
		let bRet = TRUE
	Elif(FindColors (IgnoreColor,StringToInt(sColor), S_BOTTOM, TRUE) && (GetWord() == IntToString(iChannel))) then
			let bRet = TRUE
	EndIf
	RestoreCursor ()
	RestoreCursor ()
	return bRet
EndFunction

/******************
 * Description:
 *      Collects info from the different timers
 * Return:
 *      -
 * Param:
 *      -
 * Note:
 *      Writes info in the global array gsTimers
 ******************/
String Function GetTimers ()
	var handle hWnd,
	string sLine,
	string sZoomRatio,
	string sSelectionLength,
	string sSelectionEnd,
	string sSelectionStart,
	string sTmp,
	int iRestriction,
	int iPos
	if (not IsDataWindow ()) then
		return ""
	EndIf
	SaveCursor ()
	InvisibleCursor ()
	RouteInvisibleToPc()
	SaveCursor ()
	; Search for Rate: on the bottom line in the data window
;	FindString (Getfocus(), scRate, S_BOTTOM, S_UNRestrictED)
;	NextWord()
;	NextWord()
	JAWSPageDown ()
	PriorLine()
	PriorLine()
	SetRestriction (RestrictWindow)
	JawsHome()
	; Get time displays and ratio data
	let sLine = GetToEndOfLine ()
	RestoreCursor ()
	RestoreCursor ()
	if (StringDiff (sLine, "Rate:")) then
		let sLine = StringDiff (sLine,"Rate:")
	EndIf
	let sLine = StringTrimLeadingBlanks (sLine)
;SayString(sLine)
	; Chop sLine so the SelectionStart is the first info
	let iPos = StringContains(sLine,":")
	if (iPos < 3) then
		return ""
	EndIf
	let sLine = StringChopLeft (sLine,iPos - 3)
	; Extract the zoom ratio
	let iPos = StringContains (StringReverse (sLine)," ")
	if (iPos >= 1) then
		let sZoomRatio = stringStripAllBlanks (StringRight (sLine,iPos-1))
	EndIf
	let sTmp = stringChopRight (sLine,StringContains (StringReverse (sLine),":"))
	let sTmp = stringChopRight (sLine,StringContains (StringReverse (sLine)," "))
	let sSelectionStart = StringLeft(sTmp,12)	
	let sTmp = StringChopLeft(sTmp,12)
	let sTmp = stringStripAllBlanks (sTmp)
	let sSelectionLength = StringRight(sTmp,12)
	; Chop SelectionLength from sTmp
	if (StringLength(sTmp) > 12) then
		; There is an active selection
		let sSelectionEnd = StringLeft (sTmp,12)
	EndIf
	; Return an | separated array of time fields
	; The last field is the ratio
	return sSelectionStart+"|"+sSelectionEnd+"|"+sSelectionLength+"|"+sZoomRatio
EndFunction

/******************
 * Description:
 *      Reads the number of channels from the status bar
 * Return:
 *      Number of channels
 * Param:
 *      -
 * Note:
 *      - 
 ******************/
Int Function GetNumberOfChannels ()
	var string sStatusLine, string sTmp
	let sStatusLine = GetStatusBar ()
	if StringContains (sStatusLine,scMono) then
		return 1
	ElIf StringContains(sStatusLine,scStereo) then
		return 2
	Else
		let sTmp = StringRight (StringLeft(sStatusLine,StringContains(sStatusLine,"ch.")-1),3)
		let sTmp = stringStripAllBlanks (sTmp)
		return StringToInt (sTmp)
	EndIf
EndFunction

/******************
/******************
 * Description:
 *      Converts a time string from the format hh:mm:ss to msec
 * Return:
 *      The time in msec
 * Param:
 *      -
 * Note:
 *      -
 ******************/
String Function TimeToMSec (string sTime)
	var string sTmp, int isec, int iTot
	let sTmp = StringSegment(sTime,":",1)
	let iTot = StringToInt(sTmp)*3600*1000
	let sTmp = StringSegment(sTime,":",2)
	let iTot = iTot + StringToInt(sTmp)*60*1000
	let sTmp = StringSegment(sTime,":",3)	
	let iTot = iTot + StringToInt(sTmp) * 1000
	if (StringContains(sTmp,",")) then
		; , as decimal point
		let sTmp = StringSegment(sTime,",",2)
	Else
		; . as decimal point
		let sTmp = StringSegment(sTime,".",2)
	EndIf
	let iTot = iTot + StringToInt(sTmp)
	return IntToString (iTot)
EndFunction

/******************
 * Description:
 *      Set the work mode
 * Return:
 *      -
 * Param:
 *      The desired mode
 * Note:
 *      -
 ******************/
Void Function SetMode (int iMode)
	let giMode = iMode
EndFunction
/******************
 * Description:
 *      Checks if data is selected
 * Return:
 *      True if data is selected
 * Param:
 *      -
 * Note:
 *      -
 ******************/
Int Function IsDataSelected ()
	return (GetSelectionEND () != "")
EndFunction

/******************
 * Description:
 *      Converts a time string into a more readable format
 * Return:
 *      The converted time string
 * Param:
 *      The string to be converted
 * Note:
 *      -
 ******************/
String Function ParseTimeString (string sTime)
	var string sResult, int iHours, int iMinutes, string sSeconds
	let iHours = StringToInt(StringReverse(StringSegment(StringReverse (sTime),":",3)))
	let iMinutes = StringToInt(StringReverse(StringSegment(StringReverse (sTime),":",2)))
	let sSeconds = StringReverse(StringSegment(StringReverse (sTime),":",1))
	if (iHours) then
		let sResult = sResult + IntToString(iHours) + "hours"
	EndIf
	if (iMinutes) then
		let sResult = sResult + IntToString(iMinutes) + "minutes"
	EndIf
	; Replace , with .
	let sSeconds = StringReplaceChars(sSeconds,",",".")
	; Strip leading 0
	while (StringLeft(sSeconds,1) == "0")
		let sSeconds = StringChopLeft(sSeconds,1)
	EndWhile
	if (StringLeft(sSeconds,1) == ".") then
		let sResult = sResult + " " + "0" + sSeconds + " " + "seconds"
	else
		let sResult = sResult + " " + sSeconds + " " + "seconds"
	EndIf
	return sResult
EndFunction

/******************
 * Description:
 *      Checks if the a window is an updown slider
 * Return:
 *      True if hWnd is an updown slider
 * Param:
 *      The object
 * Note:
 *      -
 ******************/
Int Function IsSFUpDownSlider (handle hWnd)
	var string sClass
	let sClass = GetWindowClass(hWnd)
	return (StringContains(sClass,"Trackbar") || StringContains(sClass,"Fader"))
EndFunction

/******************
 * Description:
 *      Returns the label connected to the slider hWnd
 * Return:
 *      The label
 * Param:
 *      The handle of the slider
 * Note:
 *      -
 ******************/
String Function GetSFUpDownSliderLabel (handle hWnd)
	var string sLabel, int iCID, string sClass,
	string sWinName
	let iCID = GetControlID(hWnd)
	let sClass = GetWindowClass(hWnd)
	let sWinName = GetWindowName (GetRealWindow (GetFocus()))
	; Label 2 before
	if (StringContains(sWinName,"Graphic EQ") 
		&& (StringContains(sClass,"sfppack2Bld314FaderClass") || StringContains(sClass,"sfppack2Bld1402FaderClass")) 
		&& (iCID >= 6006) 
		&& (iCID <= 6025)
		&& (not IsWindowVisible (GetPriorWindow(GetFocus())))) then
		; Process/EQ/Graphic
		if (giDebug) then
		SayString("Equalizer")
		EndIf
		let sLabel = GetWindowText(GetPriorWindow(GetPriorWindow (hWnd)),0)
	; Label 2 before
      	ElIf (StringContains(sWinName,"ExpressFX Equalization") && ((StringContains(sClass,"sfxpfx2Bld343FaderClass") && (iCID == 2002)) || (StringContains(sClass,"sfxpfx2Bld1402FaderClass"))))
 		|| (StringContains(sWinName,"ExpressFX Graphic EQ") && (StringContains(sClass,"sfxpfx3Bld131FaderClass") || StringContains(sClass,"sfxpfx3Bld1402FaderClass")) && (iCID != 2012))      
		|| (StringContains(sWinName,"Flange/Wah-Wah") && StringContains(sClass,"sfppack3Bld326TrackbarClass"))
		|| (StringContains(sWinName,"Multi-Tap Delay") && (iCID == 4031))
		|| (StringContains(sWinName,"Simple Delay") && ((iCID == 3016) || (iCID == 3017))
		|| (StringContains(sWinName,"Noise Reduction") && ((iCID == 3020) || (iCID == 3025))))
		|| (StringContains(sWinName,"Pitch Shift") && (iCID == 2024)) then
		; FX Favorites/ExpressFX Amplitude Modulation/Modulation/Original
		; FX Favorites/ExpressFX Equalization/Original
		; FX Favorites/ExpressFX Graphic eq
		; Effects/Flange/Wah-Wah/*
		; Effects/Delay/Multi tap/Pan 
		; Effects/Delay/Simple/Delay time
		; FX Favorites/Noise Reduction
		; FX Favorites/Pitch Shift
		if (giDebug) then
			SayString("Label 2 before")
		EndIf
		let sLabel = GetWindowText(GetPriorWindow(GetPriorWindow(hWnd)),0)
	; Label 4 before
	Elif (StringContains(sWinName,"Chorus") && StringContains(sClass,"sfxpfx2Bld343FaderClass") && (iCID == 3002))
		|| (StringContains(sWinName,"ExpressFX Amplitude Modulation") && StringContains(sClass,"sfxpfx2Bld343FaderClass") && (iCID == 4002)) then
 		; FX Favorites/ExpressFX Chorus/Chorus
		; FX Favorites/ExpressFX Amplitude Modulation/Modulation
		if (giDebug) then
			SayString("Label 4 before")
		EndIf
		let sLabel = GetWindowName(GetPriorWindow(GetPriorWindow(GetPriorWindow(GetPriorWindow(hWnd)))))
	; Label 1 after
	ElIf (StringContains(sWinName,"ExpressFX Noise Gate") && StringContains(sClass,"sfxpfx3Bld131FaderClass") && (iCID == 3002))
		|| (StringContains(sWinName,"ExpressFX Dynamics") && StringContains(sClass,"sfxpfx3Bld131FaderClass") && (iCID == 1002))
		|| (StringContains(sWinName,"Pitch Bend") && StringContains(sClass,"SfKrn_Fader_Class") && (iCID == 10200)) then
		; FX Favorites/ExpressFX/Noise Gate/Threshold
		; FX Favorites/ExpressFX Dynamics/Threshold and FX Favorites/ExpressFX/Noice gate/Threshold
		; Effect/Pitch/Bend/Range
		if (giDebug) then
			SayString("Label 1 after")
		EndIf
		let sLabel = GetWindowText(GetNExtWindow(hWnd),0)
	; Special handling for the mix window, Ctrl+m
	; Label at ID 4400 
	Elif (StringContains(sWinName,"Mix/Replace") && (GetWindowClass(hWnd) == "SfKrn_Fader_Class") && (iCID == 4404)) then
			if (giDebug) then
				SayString("Mix, label")
		EndIf
		let hWnd = FindDescendantWindow(GetParent(GetFocus()),4400)
		let sLabel = GetWindowName(hWnd)
	; Special handling for the mix window, Ctrl+m
	; Label 4 before
	Elif (StringContains(sWinName,"Mix/Replace") && (GetWindowClass(hWnd) == "SfKrn_Fader_Class") && (iCID == 4418)) then
			if (giDebug) then
				SayString("Mix, label 4 before")
		EndIf
		let sLabel = GetWindowName(GetPriorWindow(GetPriorWindow(GetPriorWindow(GetPriorWindow (hWnd)))))
	; Process/EQ/Paragraphic/Gain
	Elif (StringContains(sWinName,"Paragraphic") && ((iCID >= 3013) && (iCID <= 3016))) then
		let hWnd = FindDescendantWindow(GetParent (GetFocus()),3073)
		let sLabel = GetWindowText(hWnd,0)
	; Process/EQ/Paragraphic/Width
	Elif (StringContains(sWinName,"Paragraphic") && ((iCID >= 3041) && (iCID <= 3045))) then
		let hWnd = FindDescendantWindow(GetParent (GetFocus()),3074)
		let sLabel = GetWindowText(hWnd,0)

	; Process/EQ/Paragraphic/Center frequency
	Elif (StringContains(sWinName,"Paragraphic") && ((iCID >= 3048) && (iCID <= 3051))) then
		let hWnd = FindDescendantWindow(GetParent (GetFocus()),3075)
		let sLabel = GetWindowText(hWnd,0)
	else
		; Default handling
			if (giDebug) then
				SayString("label Default")
			EndIf
			let sLabel = GetWindowText(GetPriorWindow(hWnd),0)
	EndIf
	return sLabel
EndFunction

/******************
 * Description:
 *      Returns the value for the slider hWnd
 * Return:
 *      The value of the slider
 * Param:
 *      -
 * Note:
 *      -
 ******************/
String Function GetSFUpDownSliderValue (handle hWnd)
	var string sValue, int iCID, string sClass, string sWinName
	let iCID = GetControlID(hWnd)
	let sClass = GetWindowClass(hWnd)
	let sWinName = GetWindowName(GetRealWindow(GetFocus()))
	if ((StringContains(sWinName,"ExpressFX") && StringContains(sClass,"sfxpfx2Bld1402FaderClass") && ((iCID == 4002) || (iCID == 3002) || (iCID == 1002))) || StringContains(sClass,"sfxpfx1Bld344FaderClass") || StringContains(sClass,"sfxpfx2Bld343FaderClass")) then
		; FX Favorites/Express/FX* sliders with two values
		if (giDebug) then
			SayString("2 values")
		EndIf
		let sValue = GetWindowName(GetPriorWindow(GetPriorWindow(GetPriorWindow(hWnd))))
		let sValue = sValue + " " + GetWindowName(GetPriorWindow(GetPriorWindow(hWnd)))
		let sValue = sValue + " " + GetWindowName(GetPriorWindow(hWnd))
	; Value 1 before
	ElIf (((StringContains(sWinName,"ExpressFX Dynamics") || StringContains(sWinName,"ExpressFX Noise Gate")) && StringContains(sClass,"sfxpfx3Bld131FaderClass") && ((iCID == 3002) || (iCID == 1002)))
		|| (StringContains(sWinName,"ExpressFX Equalization") && (StringContains(sClass,"sfxpfx2Bld343FaderClass") || StringContains(sClass,"sfxpfx2Bld1402FaderClass")))
		|| (StringContains(sWinName,"Flange/Wah-Wah") && StringContains(sClass,"sfppack3Bld326TrackbarClass") && ((iCID == 1010) || (iCID == 1016) || (iCID == 1013) || (iCID == 1015))) 
		|| ((StringContains(sWinName,"Multi-Tap Delay") || StringContains(sWinName,"Simple Delay")) && ((iCID == 4031) || (iCID == 3016) || (iCID == 3017)))
		|| (StringContains(sWinName,"Channel converter") && (iCID == 10063))) then
		; FX Favorites/ExpressFX/Noise Gate/Threshold and FX Favorites/ExpressFX/Dynamics/Threshold
		; FX Favorites/ExpressFX Equalization
		; Effects/Flange/Wah-Wah/rate and Depth
		; Effects/Delay/Multi tap/Pan or Effects/Delay/Simple/Delay time
		; The slider in convert channels window
		if (giDebug) then
			SayString("Value 1 before")
		EndIf
		let sValue = GetWindowText(GetPriorWindow(hWnd),0)
	; Value 2 before
	ElIf (StringContains(sWinName,"Flange/Wah-Wah") && (sClass == "sfppack3Bld326FaderClass") && (iCID == 1003)) then
		; Effects/Flange-Wah-Wah/Dry out
		if (giDebug) then
			SayString("Value 2 before")
		EndIf
		let sValue = GetWindowText(GetPriorWindow(GetPriorWindow(hWnd)),0)
	; Value 2 after
	Elif (StringContains(sWinName,"Noise Reduction") && ((iCID == 3020) || (iCID == 3025)))
		|| (StringContains(sWinName,"Pitch Shift") && (iCID == 2024)) then
		; Noise reduction window
		; FX/Pitch Shift
		if (giDebug) then
			SayString("Value 2 after")
		EndIf
		let sValue = (GetWindowText(GetNextWindow(GetNextWindow(hWnd)),0))
	; Value 5 after
	Elif (StringContains(sWinName,"Pitch Bend") && (GetWindowClass(hWnd) == "SfKrn_Fader_Class") && (iCID == 10200)) then
	; Effect/Pitch/Bend/Range
		let sValue = GetWindowText(GetNextWindow(GetNextWindow(GetNExtWindow(GetNextWindow(GetNExtWindow(hWnd))))),0)
		let sValue = sValue + " " + GetWindowText(GetNextWindow(GetNextWindow(GetNExtWindow(GetNExtWindow(GetNextWindow(GetNextWindow(GetNExtWindow(hWnd))))))),0)
	; Special handling for the mix window, Ctrl+m
	Elif (StringContains(sWinName,"Mix/Replace") && (GetWindowClass(hWnd) == "SfKrn_Fader_Class") && (iCID == 4404)) then
		let hWnd = FindDescendantWindow(GetParent(GetFocus()),4402)
		let sValue = GetWindowText(hWnd,0)
		let hWnd = FindDescendantWindow(GetParent(GetFocus()),4403)
		let sValue = sValue + " " + GetWindowText(hWnd,0)
	; Special handling for the mix window, Ctrl+m
	Elif (StringContains(sWinName,"Mix/Replace") && (GetWindowClass(hWnd) == "SfKrn_Fader_Class") && (iCID == 4418)) then
		let sValue = GetWindowText(GetPriorWindow(GetPriorWindow(GetPriorWindow (hWnd))),0) + " " + GetWindowText(GetPriorWindow(GetPriorWindow(hWnd)),0)
	else
		; Default handling
		if (giDebug) then
			SayString("default value")
		EndIf
		let sValue = (GetWindowText(GetNextWindow(hWnd),0))
	EndIf
	return sValue
EndFunction

/******************
 * Description:
 *      Checks if hWnd is an SF spread
 * Return:
 *      True if hWnd is an SF spread
 * Param:
 *      The object to check
 * Note:
 *      -
 ******************/
Int Function IsSFSpread (handle hWnd)
	return 		(GetWindowClass (hWnd) == "SfKrn_Spread_Class") 
EndFunction

/******************
 * Description:
 *      Close all module windows
 * Return:
 *      True on success
 * Param:
 *      -
 * Note:
 *      -
 ******************/
Int Function CloseListWindows ()
	var string sApp, handle hWnd, int bRet
	let sApp = GetAppFileName ()
	if (StringContains(sApp,csForge90)) then
		let hWnd = FindTopLevelWindow ("Forge9.Class.Doc", "")
	ElIf (StringContains(sApp,csForge100)) then
		let hWnd = FindTopLevelWindow ("Forge10.Class.Doc", "")
	ElIf (StringContains(sApp,csForge110)) then
		let hWnd = FindTopLevelWindow ("Forge10.Class.Doc", "")
	EndIf
	SendMessage(hWnd, WM_CLOSE,0 , 0)
	if (not hWnd) then
		return FALSE
	EndIf
	while (hWnd)
		if (StringContains(sApp,csForge90)) then
			let hWnd = FindTopLevelWindow ("Forge9.Class.Doc", "")
		ElIf (StringContains(sApp,csForge100)) then
			let hWnd = FindTopLevelWindow ("Forge10.Class.Doc", "")
		ElIf (StringContains(sApp,csForge110)) then
			let hWnd = FindTopLevelWindow ("Forge10.Class.Doc", "")
		EndIf
		SendMessage(hWnd, WM_CLOSE,0 , 0)
	EndWhile
	return TRUE
EndFunction

Function DebugMessage (string sMessage, int bBox)
	if (giDebug) then
		if (bBox) then
		MessageBox(sMessage)
		else
			SayMessage(OT_NO_DISABLE,sMessage,sMessage)
		EndIf
	EndIf
EndFunction

int Function WaitforWindow (string sClass, int iCtrlID)
	var int bRet, int iLoops,
	int iCurCtrlID, string sCurClass
	while ((iLoops < DEADLOCK) && (not bRet))
		let sClass = GetWindowClass (GetFocus ())
		let iCtrlID = GetControlID (GetFocus ())
		let iLoops = iLoops + 1
		if ((iCtrlID == iCurCtrlID) && (sClass == sCurClass)) then
			let bRet = TRUE
		EndIf
	EndWhile
	return bRet
EndFunction

Int Function GetCustomControlBase()
	Var
		Int iReturnValue,
		Int iJawsVersion,
		int iIndex
        
	Let iJawsVersion = GetJFWVersion() / 100000
	let iIndex = 1    
	while (stringSegment(gsCustomControlBase,"|",iIndex))
		if (iJawsVersion == StringToInt(StringSegment(StringSegment(gsCustomControlBase,"|",iIndex),"#",1))) then
			let iReturnValue = StringToInt(StringSegment(StringSegment(gsCustomControlBase,"|",iIndex),"#",2))
		EndIf
		let iIndex = iIndex + 1
	EndWhile
	if (not iReturnValue) then
		MessageBox("Error in GetCustomControlBase")
	EndIf

	Return iReturnValue
EndFunction
