include "hjconst.jsh"
include "hjglobal.jsh"
include "forge.jsh"
include "forge.jsm"
use "forgelib.jsb"
use "forgetrack.jsb"

/******************
 * Description:
 *      Opens the markers and regions list
 * Return:
 *      -
 * Param:
 *      -
 * Note:
 *      Sends an message to simulate a choice from the menu
 ******************/
Function OpenMarkersAndRegionsList ()
	var string sApp,
	int iLoops
	let sApp = GetAppFileName ()
	if (StringContains(sApp,csForge90)) then
		SendMessage (GetAppMainWindow(GetFocus()),WM_COMMAND, MENU_MARKERSLIST_FORGE90,0)
	ElIf (StringContains(sApp,csForge100)) then
		SendMessage (GetAppMainWindow(GetFocus()),WM_COMMAND, MENU_MARKERSLIST_FORGE100,0)
	ElIf (StringContains(sApp,csForge110)) then
		SendMessage (GetAppMainWindow(GetFocus()),WM_COMMAND, MENU_MARKERSLIST_FORGE110,0)
	EndIf
	; Delay until the list is open and one item is highlighted
	WaitForWindow (csSFSpread, ciMarkersAndRegionsListCID)
	Pause ()
EndFunction

/******************
 * Description:
 *      Extracts info about markers and regions from the markers and regions list
 * Return:
 *      An array of markers or regions
 * Param:
 *      -
 * Note:
 *      Opens the markers and regions list and loops through the list to collect info
 ******************/
String Function GetMarkers ()
	var string sMarkersList, string sStart, string sEnd, string sMarkerNo, string sMarkerName, string sMarkerStart, string sMarkerEnd, string sTmp,
	int iLoops, int bRet, int iMarkerNO, int bSpeechOn

	; Force an extra update of the timers
	let gsTimers = GetTimers ()

	; Close any module windows
	CloseListWindows ()

	; Moving in the markers and regions list may cause the cursor to be moved in the file and data beeing selected
	; Save the current pos in the file
	let sStart = GetSelectionStart()
	; If selection end has a value then data is selected
	let sEnd = GetSelectionEnd ()

	; Get the status of the speech
	let bSpeechOn = SpeechInUse ()
	; Mute synth to prevent unwanted speech
	if (bSpeechOn) then
		SpeechOff ()
		Pause ()
	EndIf

	; Open the markers and regions list
	OpenMarkersAndRegionsList ()
	let iMarkerNO = 1
	SelectFirstInMarkersList ()
	; Loop through the list, max 100 markers
	while (GetWindowText (GetFocus(),1) && (iLoops < MAXNOMARKERSANDTRACKS))
		Delay(1,true)
		RouteInvisibleToPc ()
		InvisibleCursor ()
		if(FindFirstAttribute (ATTRIB_HIGHLIGHT, true)) then
			; Get the marker NO
			let sMarkerNO = IntToString(iMarkerNO)
			let iMarkerNO = iMarkerNO + 1
			; Get the marker name
			let sMarkerName = GetChunk ()
			NextChunk ()
			let sTmp = GetChunk ()
			; Get marker start and end
			let sMarkerStart = StringSegment (sTmp," ",1)
			let sMarkerEnd = StringSegment (sTmp," ",2)
			let sMarkersList = sMarkersList + sMarkerNO+"#"+sMarkerName+"#"+sMarkerStart + "#" + sMarkerEnd +"|"
		EndIf
		PcCursor ()
		NextLine ()
		let iLoops = iLoops + 1
	EndWhile
	let sMarkersList = StringChopRight(sMarkersList,1)
	let bRet = CloseListWindows ()
	Pause ()
	if (not bRet) then
	; Failed to close the window
		DebugMessage("Failed to close markers and regions list",1)
	EndIf
	let gsTimers = GetTimers ()
	if (StringCompare (GetSelectionStart(),sStart) != 0) then
		; The cursor has moved when looping through the markers list
		; This is a bug in SF
		TypeKey("Ctrl+g")
		WaitForWindow ("edit",1000)
		TypeString(sStart)
		{Enter}
 	EndIf		
	Pause ()
	; Set back selection if something was selected by the user
	if (sEnd) then
		{Ctrl+Shift+d}
		WaitForWindow ("edit",1000)
		TypeString(sStart)
		{Tab}
		TypeString(sEnd)
		{Enter}
	else
		let gsTimers = GetTimers()
		if (GetSelectionEnd()) then
			; A selection has occured, this is a bug in SF
			{s}
		EndIf
	EndIf
	Pause ()
	; Turn on speech if it was on before the call to this funciton
	if (bSpeechOn) then
		SpeechOn ()
	EndIf
	; Force an extra update of the timers
	let gsTimers = GetTimers ()
	return sMarkersList
EndFunction

; Returns a string describing the data on the Braille line
String Function GetBrailleMarkersLine ()
	var string sLoc, string sBrailleMarkersLine, string sTot, string sMarkers, string smarker,
	int iTmp, int iMSecPerCell, int iTot, int iBCell, int iMarker, int iLoc
	let sTot = TimeToMSec(GetTotalTime ())
	let iTot = StringToInt (sTot)
	let iMSecPerCell = (iTot) / BrailleGetCellCount ()
	let iBCell = 1
	let sBrailleMarkersLine = ""
	while (iBCell < (BrailleGetCellCount ()+1))
		let iMarker = 1
		let sMarkers = ""
		While (GetMarkerByIndex(gsmarkers,iMarker))
			let sMarker = GetMarkerByIndex(gsMarkers,iMarker)
			let sLoc = GetMarkerStart(sMarker)
			let iLoc = StringToInt (TimeToMsec(sLoc))
			let iTmp = ((iLoc) / iMSecPerCell) + 1
			if (iTmp == (BrailleGetCellCount () + 1)) then
				let iTmp = BrailleGetCellCount ()
			EndIf
			if (sLoc && (iTmp == iBCell)) then
				let sMarkers = sMarkers + IntToString(iMarker) +"#"
			EndIf
			let iMarker = iMarker + 1
		EndWhile
		let sMarkers = StringChopRight(sMarkers,1)
		let sBrailleMarkersLine = sBrailleMarkersLine + sMarkers + "|"
		let iBCell = iBCell + 1
	EndWhile
	let sBrailleMarkersLine = StringChopRight(sBrailleMarkersLine,1)
	return sBrailleMarkersLine
EndFunction

Void Function UpdateMarkers ()
	let gsMarkers = GetMarkers ()
	let gsBrailleMarkersLine = GetBrailleMarkersLine ()
EndFunction

; Return the marker object with the index iIndex from the marker array sMarkers
String Function GetMarkerByIndex (string sMarkers, int iIndex)
	return StringSegment(sMarkers,"|",iIndex)
EndFunction

; Returns the name from the marker object sMarker
String Function GetMarkerName (string sMarker)
	return StringSegment(sMarker,"#",2)
EndFunction

; Returns the start time from the marker object sMarker
String Function GetMarkerStart (string sMarker)
	return StringSegment(sMarker,"#",3)
EndFunction

; Returns the end time from the marker object sMarker
String Function GetMarkerEnd (string sMarker)
	return StringSegment(sMarker,"#",4)
EndFunction

; List all markers
Script ListMarkers ()
	var string sMarkersList,
	int iIndex, int iSel, 
	string sMarker, string sName, string sList, string sLoc,
	handle hWnd
	let hWnd = GetFocus ()
	let sMarkersList = GetMarkers ()
	let iIndex = 1
	while (GetMarkerByIndex(sMarkersList,iIndex))
		let sMarker = GetMarkerByIndex(sMarkersList,iIndex)
		let sName = GetMarkerName(sMarker)
		if (IsRegion(iIndex)) then
			let sName = msgRegion + ": "+ sName
		Else
			let sName = msgMarker + ": " + sName
		EndIf
		let iIndex = iIndex + 1
		if (stringStripAllBlanks (sName)) then
			let sList = sList + sName + "\007"
		EndIf
	EndWhile
	let sList = StringChopRight(sList,1)
	let iSel = DlgSelectItemInList (sList, msgGotoMarker,FALSE, 1)
	SetFocus (hWnd)
	if (iSel > 0) then
		MoveToMarkerByIndex(iSel)
	EndIf
EndScript

; Move to marker iMarker
; Imarker is a index into the gsMarkersList
Function MoveToMarkerByIndex (int iMarker)
	var string sLoc, int iLoc, string sMarker
	let sMarker = GetMarkerByIndex (gsMarkers,iMarker)
	let sLoc = GetMarkerStart (sMarker)
	{Ctrl+g}
	; Wait until the goto dialog is open
	WaitForWindow ("edit",1000)
	TypeString (sLoc)
	{Enter}
EndFunction

Int Function SelectMarkerFromMenu (string sMarkers)
	var int iSel, string sList, int iIndex, int iIndexInMarkersList, string sMarker, string sMarkerName
	let iIndex = 1
	while (StringSegment (sMarkers,"#",iIndex))
		let iIndexInMarkersList = StringToInt(StringSegment(sMarkers,"#",iIndex))
		let sMarker = GetMarkerByIndex (gsMarkers,iIndexInMarkersList)
		let sMarkerName = GetMarkerName (sMarker)
		if (StringContains  (sMarkerName,"Marker")) then
			let sMarkerName = StringDiff (sMarkerName,"Marker")
		EndIf	
		if (StringContains (sMarkerName,"Region")) then
			let sMarkerName = StringDiff (sMarkerName,"Region")
		EndIf	

		if (IsRegion(iIndexInMarkersList)) then
			let sMarkerName = msgRegion + ": " + sMarkerName
		Else
			let sMarkerName = msgMarker + ": "+ sMarkerName
		EndIf
		let sList = sList + sMarkerName + "\007"
		let iIndex = iIndex + 1
	EndWhile
	let iSel = DlgSelectItemInList (sList, msgSelectMarker, FALSE, 1)
	if (iSel > 0) then
		return StringSegment(sMarkers,"#",iSel)
	else
		return 0
	EndIf
EndFunction

; Bring up a menu with actions to perform on a marker marker
Void Function ExecuteMarkerAction (int iMarker)
	var string sLoc, int iLoc, string sMarker,
	string sMarkerText, string sMarkerTextOld,
	string sMarkerName, string sMarkerStart, string sMarkerEnd, string sMarkerNO, string sTmp, string sMarkersList,
	int iLoops,
	int iNumMarkers, int iPos, int bFound,
	int iSel, int bSpeechOn
	let sMarker = GetMarkerByIndex(gsMarkers,iMarker)
	let sMarkerName = GetMarkerName(sMarker)
	let bSpeechOn = SpeechInUse ()
	if (bSpeechOn) then
		SpeechOff ()
	EndIf
	; Open the marker and regions list
	OpenMarkersAndRegionsList ()
	SelectFirstInMarkersList ()
	let iLoops = 1
	while (iLoops < iMarker)
		Delay (1,TRUE)
		NextLine ()
		let iLoops = iLoops + 1
	EndWhile
	Pause ()
	JawsCursor ()
	RouteJawsToPc ()
	FindFirstAttribute (ATTRIB_HIGHLIGHT, TRUE)
	if (sMarkerName == GetChunk ()) then
		RightMouseButton ()
		let bUpdateMarkers = TRUE
		let gbClosed = FALSE
	EndIf
	PcCursor ()
	Pause ()
	if (bSpeechOn) then
		SpeechOn ()
	EndIf
EndFunction

Void Function ExecuteRegionAction (int iMarker)
	var string sLoc, int iLoc, string sMarker,
	string sMarkerText, string sMarkerTextOld,
	string sMarkerName, string sMarkerStart, string sMarkerEnd, string sMarkerNO, string sTmp, string sMarkersList,
	int iLoops,
	int iNumMarkers, int iPos, int bFound, int bSpeechOn,
	handle hWnd, int iSel
	let sMarker = GetMarkerByIndex(gsMarkers,iMarker)
	let sMarkerName = GetMarkerName(sMarker)
	let bSpeechOn = SpeechInUse ()
	if (bSpeechOn) then
		SpeechOff ()
	EndIf
	; Open the markers and regions list
	OpenMarkersAndRegionsList ()
	SelectFirstInMarkersList ()
	let iLoops = 1
	while (iLoops < iMarker)
		delay (1,true)
		NextLine ()
	EndWhile 
	Pause ()
	JawsCursor ()
	RouteJawsToPc ()
	FindFirstAttribute(ATTRIB_HIGHLIGHT,TRUE)
	if (sMarkerName == GetChunk ()) then
		RightMouseButton ()
		let bUpdateMarkers = TRUE
		let gbClosed = FALSE
	EndIf
	PcCursor ()
	if (bSpeechOn) then
		SpeechOn ()
	EndIf
EndFunction

script DeleteTrackOrMarker ()
	var string sMarkerName, string sTrackNO,
	int bDeleted
	if (giMode == MARKERS) then
		let gsMarkers = GetMarkers ()
		let sMarkerName = GetCurrentMarker ()
		if (sMarkerName) then
			let bDeleted = DeleteMarker(sMarkerName)
			if (bDeleted) then
				SayMessage (OT_JAWS_MESSAGE,sMarkerName + " " + msgDeleted)
			else
				SayMessage (OT_JAWS_MESSAGE,msgCouldNotDeleteMarker)
			EndIf
		else
			SayString(msgNoMarkerOrRegion)
		EndIf
	ElIf (giMode == TRACKS) then
		let gsTracks = GetTracks ()
		let sTrackNO = GetCurrentTrack ()
		if (sTrackNO) then
			let bDeleted = DeleteTrack (sTrackNO)
			if (bDeleted) then
				SayMessage (OT_JAWS_MESSAGE,msgTrack + sTrackNo + " " + msgDeleted)
			else
				SayMessage (OT_JAWS_MESSAGE,msgCouldNotDeleteTrack)
			EndIf
		else
			SayString(msgNoTrack)
		EndIf
	else
		SayMessage (OT_JAWS_MESSAGE, msgCommandNotValid, msgCommandNotValid)
	EndIf
EndScript

Int Function DeleteMarker (string sMarker)
	var string sEnd, string sStart, string sMarkerText, int bSpeechOn,
	int iLoops, int bFound
	; Force an extra update of the timers
	let gsTimers = GetTimers ()

	; Moving in the markers and regions list may cause the cursor to be moved in the file and data beeing selected
	; Save the current pos in the file
	let sStart = GetSelectionStart()
	; If selection end has a value then data is selected
	let sEnd = GetSelectionEnd ()
	; Get the status of the speech
	let bSpeechOn = SpeechInUse ()
	if (bSpeechOn) then
		SpeechOff ()
	EndIf
	Pause ()

	; Open the marker and regions list
	OpenMarkersAndRegionsList ()
	SelectFirstInMarkersList ()
	let iLoops = 0
	while (not bFound && GetWindowText (GetFocus (),1) && (iLoops < MAXNOMARKERSANDTRACKS))
		Delay (1)
		InVisibleCursor ()
		if (FindFirstAttribute (ATTRIB_HIGHLIGHT, GetFocus())) then
			;	NextChunk ()
			let sMarkerText = GetChunk ()
			if (StringContains (sMarkerText,sMarker) && StringContains (sMarker,sMarkerText)) then
				let bFound = 1
			EndIf
		EndIf
		let iLoops = iLoops + 1
		PcCursor ()
		if (not bFound) then
			NextLine ()
		EndIf
	EndWhile
	if (bFound) then
		{Delete}
		Pause ()
	EndIf
	PerformScript CloseWindow ()
	Pause ()
	; Force an extra update of the timers
	let gsTimers = GetTimers ()
	if (StringCompare(GetSelectionStart(),sStart) != 0) then
		; The cursor has moved when looping through the markers list
		; This is a bug in SF
		TypeKey("Ctrl+g")
		TypeString(sStart)
		{Enter}
 	EndIf		
	Pause ()
	; Set back selection if something was selected by the user
	if (sEnd) then
		{Ctrl+Shift+d}
		TypeString(sStart)
		{Tab}
		TypeString(sEnd)
		{Enter}
	else
		let gsTimers = GetTimers()
		if (GetSelectionEnd()) then
			; A selection has occured, this is a bug in SF
			{s}
		EndIf
	EndIf
	Pause ()
	if (bSpeechOn) then
		SpeechOn()
	EndIf

	if (bFound) then
		UpdateMarkers ()
	EndIf
	pause ()
	return bFound
EndFunction

Script SpeakMarker ()
	var string sMarkerName
	let gsMarkers = GetMArkers ()
	let sMarkerName = GetCurrentMarker ()
	if (sMarkerName) then
		SayString(sMArkerName)
	else
		SayString(msgNoMarkerOrRegion)
	EndIf
EndScript

String Function GetCurrentMarker ()
	var string sTime, int iIndex, int bFound, string sMarker, string sLoc
	if (IsDataWindow ()) then
		let sTime = GetSelectionStart ()
		let gsMarkers = GetMarkers ()
		let iIndex = 1
		While (GetMarkerByIndex(gsMarkers,iIndex) && (not bFound))
			let sMarker = GetMarkerByIndex(gsMarkers,iIndex)
			let sLoc = GetMarkerStart(sMarker)
			if (StringContains(sLoc,sTime)) then
				let bFound = 1
			EndIf
			let iIndex = iIndex + 1
		EndWhile
		if bFound then
			return GetMarkerName(sMarker)
		else
			return ""
		EndIf
	EndIf
	return 0
EndFunction

Script MarkerContextMenu ()
	var string sTime, int iIndex, int bFound, string sMarker, string sLoc
	if (IsDataWindow ()) then
		let sTime = GetSelectionStart ()
		let gsMarkers = GetMarkers ()
		let iIndex = 1
		While (GetMarkerByIndex(gsMarkers,iIndex) && (not bFound))
			let sMarker = GetMarkerByIndex (gsMarkers,iIndex)
			let sLoc = GetMarkerStart (sMarker)
			if (StringContains(sLoc,sTime)) then
				let bFound = 1
			EndIf
			let iIndex = iIndex + 1
		EndWhile
		if (bFound) then
			ExecuteMarkerAction (StringToInt(StringSegment(sMarker,"#",1)))
		else
			SayMessage(OT_JAWS_MESSAGE,msgNoMarkerOrRegion,msgNoMarkerOrRegion)
		EndIf
	EndIf
EndScript

/******************
 * Description:
 *      Checks if a marker is a region
 * Return:
 *      True if the marker is a region
 * Param:
 *      A index in the marker and regions info array
 * Note:
 *      -
 ******************/
Int Function IsRegion (int iMarker)
	var string sMarker
	let sMarker = StringSegment(gsMarkers,"|",iMarker)
	if (sMarker) then
		if (StringSegment(sMarker,"#",4)) then
			return TRUE
		EndIf
	EndIf
	Return FALSE
EndFunction


Function SelectFirstInMarkersList ()
	; Select the first object in the markers and regions list
	{Home}
	{LeftArrow}
	Pause ()
EndFunction

