include "hjglobal.jsh"
include "hjconst.jsh"
include "forge.jsh"
include "forge.jsm"

/******************
 * Description:
 *      Finds the peak-meters window handle
 * Return:
 *      The peak-meters window handle
 * Param:
 *      -
 * Note:
 *      - 
 ******************/
HANDLE Function GetPeakWindow ()
	var string sAppName,
	handle hWnd
	let hWnd = FindWindow (GetAppMainWindow(GetFocus()), "", csPeakWndTitle)
	return hWnd
EndFunction

/******************
 * Description:
 *      Finds the peak-meters window handle in the record dialog
 * Return:
 *      The peak-meters window handle
 * Param:
 *      -
 * Note:
 *      - 
 ******************/
HANDLE Function GetRecordPeakWindow ()
	var string sAppName,
	handle hWnd
	let sAppName = GetAppFileName ()
if (sAppName == csForge90) then
		let hWnd = FindWindow (GetRealWindow(GetFocus()), csSF90RecordPeakWndClass, "")
	ElIf (sAppName == csForge100) then
		let hWnd = FindWindow (GetRealWindow(GetFocus()), csSF10RecordPeakWndClass, "")
	else
		return 0
	EndIf
	return hWnd
EndFunction

/******************
 * Description:
 *      Reads the text from all peak meters in the peak meters window
 * Return:
 *      The peak meter values as an array
 * Param:
 *      -
 * Note:
 *      -
 ******************/
String Function GetPeakMeters ()
	var handle hWnd,
	int iRestriction,
	string sPeakMeters,
	int iNOChannels, int iIndex, int iWidth, int iCurWidth
	if (IsDataWindow ()) then
		let hWnd = GetPeakWindow ()
	ElIf(IsRecordDlg()) then
		let hWnd = GetRecordPeakWindow ()
	else
		return 0
	EndIf
	let iNOChannels = GetNumberOfChannels ()
	if (not IsRecordDlg ()) then
		let iCurWidth = GetWindowRight (hWnd) - GetWindowLeft (hWnd)
		let iWidth = ((iNOChannels + iNOChannels % 2) * 32) 
		if (((iWidth - iCurWidth) * (iWidth - iCurWidth)) > 25) then
			PerformScript AdjustPeakMetersWindow ()
		EndIf
	EndIf
	SaveCursor ()
	InvisibleCursor ()
	let iRestriction = GetRestriction()
	let iNOChannels = GetNumberOfChannels ()
	SaveCursor ()
	MoveToWindow (hWnd)	
	SetRestriction(RestrictWindow)
	NextLine ()
	JawsHome ()
	while (iIndex < iNOChannels)
		let sPeakMeters = sPeakMeters + GetChunk ()
		if (GetColorBackground () == 255) then
		; The background is read, we had a peak value
				let sPeakMeters = sPeakMeters + "#1"
		else
				let sPeakMeters = sPeakMeters + "#0"
		EndIf
		let sPeakMeters = sPeakMeters + "#" + IntToString (GetCursorCol()) + "#" + IntToString(GetCursorRow())
		if (iIndex < (iNOChannels-1)) then
			let sPeakMeters = sPeakMeters + "|"
		EndIf
		NextChunk ()
		let iIndex = iIndex + 1
	EndWhile
	SetRestriction(iRestriction)
	RestoreCursor ()
	RestoreCursor ()
	return sPeakMeters
EndFunction

; Lists all the peak meters
Script ListPeakMeters ()
	var int iIndex, string sList, int iSel, string sPeakMeter,
	handle hWnd
	let hWnd = GetFocus ()
	let iIndex = 1
	let gsPeakMeters = GetPeakMeters ()
	while (GetPeakMeter(gsPeakMeters,iIndex))
		let sPeakMeter = GetPeakMeter (gsPeakMeters,iIndex)
		let sList = sList + GetPeakMeterValue(sPeakMeter) + "\007"
		let iIndex = iIndex + 1
	EndWhile
	let iSel = DlgSelectItemInList (sList, msgSelectPeakMeterToClear, FALSE, 1)
	SetFocus (hWnd)
	if (iSel > 0) then
		ClearPeakMeter (iSel)
	EndIf
EndScript

; Clear peak meter N
Void Function ClearPeakMeter (int n)
	var handle hWnd,	string stmp
	let hWnd = FindWindow (GetAppMainWindow(GetFocus()), "", "Channel Meters")
	SaveCursor ()
	JawsCursor ()
	SaveCursor ()
	let stmp = GetPeakMeter (gsPeakMeters,n)
	MoveTo (GetPeakMeterX(sTmp), GetPeakMeterY(sTmp))
	LeftMouseButton ()
	RestoreCursor ()
	RestoreCursor ()
	SayString(FormatString(msgPeakMeterNOCleared,IntToString(n)))
	{Alt+0}
EndFunction


String Function GetPeakMeter (string sPeakMeters, int nIndex)
	return StringSegment(sPeakMeters,"|",nIndex)
EndFunction


String Function GetPeakMeterValue (string sPeakMeter)
	return StringSegment(sPeakMeter,"#",1)
EndFunction

Int Function GetPeakMeterX (string sPeakMeter)
	return StringToInt(StringSegment(sPeakMeter,"#",3))
EndFunction

Int Function GetPeakMeterY (string sPeakMeter)
	return StringToInt(StringSegment(sPeakMeter,"#",4))
EndFunction

/******************
 * Description:
 *      Checks if clipping has occured
 * Return:
 *      True if clipping has occured in any channel
 * Param:
 *      -
 * Note:
 *      -
 ******************/
Int Function PeakClipping (string sPeakList)
	var int iIndex, int bClipping
	let iIndex = 1
	while (StringSegment(sPeakList,"|",iIndex))
		if (StringSegment(StringSegment(sPeakList,"|",iIndex),"#",2) == "1") then
				let bClipping = TRUE
		EndIf
		let iIndex = iIndex + 1
	EndWhile
	return bClipping
EndFunction

/******************
 * Description:
 *      Toggle clipping alarm on or off
 * Return:
 *      -
 * Param:
 *      -
 * Note:
 *      -
 ******************/
Script ToggleClippingCheck ()
	if (giClipping) then
		let giClipping = FALSE
		SayString(msgCheckClippingOff)
	else
		let giClipping = TRUE
		SayString(msgCheckClippingOn)
	EndIf
EndScript

