include "forge.jsh"
include "hjconst.jsh"
use "forgelib.jsb"

/******************
 * Description:
 *      Opens the CD tracks list
 * Return:
 *      -
 * Param:
 *      -
 * Note:
 *      -
 ******************/
Function OpenTracksList ()
	var int iLoops, string sApp
	let sApp = GetAppFileName ()
	if (sApp == csForge100) then
		SendMessage(GetAppMainWindow(GetFocus()),WM_COMMAND, MENU_TRACKSLIST_FORGE100,0)
	else
		SendMessage(GetAppMainWindow(GetFocus()),WM_COMMAND, MENU_TRACKSLIST_FORGE110,0)
	EndIf
	; Delay until the list is open and one item is highlighted
	WaitForWindow (csSFSpread, ciMarkersAndRegionsListCID)
	Pause ()
EndFunction

/*******************
 * Description:
 *      Extracts info about CD tracks from the tracks list
 * Return:
 *      An array of CD tracks
 * Param:
 *      -
 * Note:
 *      Opens the tracks list and loops through the list to collect info
 ******************/
string Function GetTracks ()
	var string sTrackName, string sTrackStart, string sTrackEnd, string sTrackNO, string sStart, string sEnd,
	string sTracksList, string sTmp,
	int iLoops, int bRet, int bSpeechOn
	; Force an extra update of the timers
	let gsTimers = GetTimers ()
	CloseListWindows ()
	; Moving in the markers and regions list may cause the cursor to be moved in the file and data beeing selected
	; Save the current pos in the file
	let sStart = GetSelectionStart()
	; If selection end has a value then data is selected
	let sEnd = GetSelectionEnd ()
	; Get the status of the speech
	let bSpeechOn = SpeechInUse ()
	; Mute synth to prevent unwanted speech
	if (bSpeechOn) then
		SpeechOff ()
		Pause ()
	EndIf

	; Open the track list
	OpenTracksList ()
	SelectFirstInMarkersList ()
	; Loop through the list, max 100 tracks
	while (GetWindowText (GetFocus (),1) && (iLoops < MAXNOMARKERSANDTRACKS))
		Delay(1,TRUE)
		RouteInvisibleToPc ()
		InvisibleCursor ()
		JawsHome ()
		NextChunk ()
		let sTrackNO = GetChunk ()
		NextChunk ()
		let sTmp = GetChunk ()
		let sTrackStart = StringSegment(sTmp," ",2)
		let sTrackEnd = StringSegment(sTmp," ",3)
		let sTracksList = sTracksList + sTrackNO+"#"+sTrackStart+"#"+sTrackEnd + "|"
		PcCursor ()
		NextLine ()
		let iLoops = iLoops + 1
	EndWhile
	let sTracksList = StringChopRight(sTracksList,1)
	let bRet = CloseListWindows ()
	Pause ()
	if (not bRet) then
		DebugMessage("Failed to close tracks list",1)
	EndIf
	let gsTimers = GetTimers ()
	if (StringCompare(GetSelectionStart(),sStart) != 0) then
		; The cursor has moved when looping through the markers list
		; This is a bug in SF
		TypeKey("Ctrl+g")
		WaitForWindow ("edit", 1000)
		TypeString(sStart)
		{Enter}
	EndIf		
	Pause ()
	; Set back selection if something was selected by the user
	if (sEnd) then
		{Ctrl+Shift+d}
		WaitForWindow ("edit", 1000)
		TypeString(sStart)
		{Tab}
		TypeString(sEnd)
		{Enter}
	else
		pause () 
		pause ()
		let gsTimers = GetTimers()
		if (GetSelectionEnd()) then
			; A selection has occured, this is a bug in SF
			{s}
		EndIf
	EndIf
	Pause ()
	; Turn on speech if it was on before the call to this funciton
	if (bSpeechOn) then
		SpeechOn ()
	EndIf
	; Force an extra update of the timers
	let gsTimers = GetTimers ()
	return sTracksList
EndFunction

String Function GetBrailleTracksLine ()
	var string sLoc, string sBrailleTracksLine, string sTot, string sTracks,
	int iTmp, int iMSecPerCell, int iTot, int iBCell, int iTrack, int iLoc
	let sTot = TimeToMSec(GetTotalTime ())
	let iTot = StringToInt (sTot)
	let iMSecPerCell = (iTot) / BrailleGetCellCount ()
	let iBCell = 1
	let sBrailleTracksLine = ""
	while (iBCell < (BrailleGetCellCount ()+1))
		let iTrack = 1
		let sTracks = ""
		While (StringSegment (gsTracks,"|",iTrack))
			let sLoc = StringSegment(StringSegment(gsTracks,"|",iTrack),"#",2)
			let iLoc = StringToInt (TimeToMsec(sLoc))
			let iTmp = ((iLoc) / iMSecPerCell) + 1
			if (iTmp == (BrailleGetCellCount () + 1)) then
				let iTmp = BrailleGetCellCount ()
			EndIf
			if (sLoc && (iTmp == iBCell)) then
				let sTracks = sTracks + IntToString(iTrack) +"#"
			EndIf
			let iTrack = iTrack + 1
		EndWhile
		let sTracks = StringChopRight(sTracks,1)
		let sBrailleTracksLine = sBrailleTracksLine + sTracks + "|"
		let iBCell = iBCell + 1
	EndWhile
	let sBrailleTracksLine = StringChopRight(sBrailleTracksLine,1)
	return sBrailleTracksLine
EndFunction

Int Function SelectTrackFromMenu (string sTracks)
EndFunction

; Moves to the specified track marker
; iTrack is a pointer into gsTracks
Void Function MoveToTrackByIndex (int iTrack)
	var string sLoc, int iLoc, string sTrack
	let sTrack = GetTrackByIndex(gsTracks,iTrack)
	let sLoc = GetTrackStart(sTrack)
	{Ctrl+g}
	WaitForWindow ("edit", 1000)
	TypeString(sLoc)
	{Enter}
EndFunction

Void Function UpdateTracks ()
	let gsTracks = GetTracks () 
	let gsBrailleTracksLine = GetBrailleTracksLine ()
EndFunction

Int Function DeleteTrack (string sTrackNO)
	var string sNO, string sStart, string sEnd,
	int iLoops, int bFound, int bSpeechOn
	; Force an extra update of the timers
	let gsTimers = GetTimers ()
	; Moving in the markers and regions list may cause the cursor to be moved in the file and data beeing selected
	; Save the current pos in the file
	let sStart = GetSelectionStart()
	; If selection end has a value then data is selected
	let sEnd = GetSelectionEnd ()
	; Get the status of the speech
	let bSpeechOn = SpeechInUse ()
	if (bSpeechOn) then
		SpeechOff ()
	EndIf
	Pause ()
	; Open the track list
	OpenTracksList ()
	Pause ()
	let iLoops = 0
	SelectFirstInMarkersList ()
	; Loop through the list, max 100tracks
	while ((not StringContains(GetLine(),"Track")) && (not bFound) && (iLoops < MAXNOMARKERSANDTRACKS)) 
		RouteInvisibleToPc ()
		InvisibleCursor ()
		FindFirstAttribute (ATTRIB_HIGHLIGHT, TRUE)
		JawsHome ()
		; Get theTrack NO
		let sNO = GetChunk ()
		PcCursor ()
		if (StringContains(sNo,STrackNo)) then
			let bFound = TRUE
		EndIf
		if (not bFound) then
			NextLine ()
		EndIf
		let iLoops = iLoops + 1
	EndWhile
;MessageBox(IntToString(iLoops))
	if (bFound) then
		{Delete}
		Pause ()
	EndIf

	PerformScript CloseWindow ()
	Pause ()
	let gsTimers = GetTimers ()
	if (StringCompare(GetSelectionStart(),sStart) != 0) then
		; The cursor has moved when looping through the markers list
		; This is a bug in SF
		TypeKey("Ctrl+g")
		WaitForWindow ("edit", 1000)
		TypeString(sStart)
		{Enter}
 	EndIf		
	Pause ()
	; Set back selection if something was selected by the user
	if (sEnd) then
		{Ctrl+Shift+d}
		WaitForWindow ("edit", 1000)
		TypeString(sStart)
		{Tab}
		TypeString(sEnd)
		{Enter}
	else
		Pause ()
		let gsTimers = GetTimers()
		if (GetSelectionEnd()) then
			; A selection has occured, this is a bug in SF
			{s}
		EndIf
	EndIf
	Pause ()
	; Turn on speech if it was on before the call to this funciton
	if (bSpeechOn) then
		SpeechOn ()
	EndIf
	if (bFound) then
		UpdateTracks ()
	EndIf
	return bFound
EndFunction

string Function GetCurrentTrack ()
	var string sTime, int iIndex, int bFound, string sTrack, string sLoc
	if (IsDataWindow ()) then
		let sTime = GetSelectionStart ()
		let gsTracks = GetTracks ()
		let iIndex = 1
		While (GetTrackByIndex(gsTracks,iIndex) && (not bFound))
			let sTrack = GetTrackByIndex (gsTracks,iIndex)
			let sLoc = GetTrackStart (sTrack)
			if (StringContains(sLoc,sTime)) then
				let bFound = 1
			EndIf
			let iIndex = iIndex + 1
		EndWhile
		if bFound then
			return GetTrackName(sTrack)
		else
			return ""
		EndIf
	EndIf
	return 0
EndFunction


; Return the track object with the index iIndex from the track array sTracks
String Function GetTrackByIndex (string sTracks, int iIndex)
	return StringSegment(sTracks,"|",iIndex)
EndFunction

; Returns the name from the track object sTrack
String Function GetTrackName (string sTrack)
	return StringSegment(sTrack,"#",1)
EndFunction

; Returns the start time from the track object sTrack
String Function GetTrackStart (string sTrack)
	return StringSegment(sTrack,"#",2)
EndFunction

; Returns the end time from the track object sTrack
String Function GetTrackEnd (string sTrack)
	return StringSegment(sTrack,"#",3)
EndFunction


